/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

// <GFI date="2001/03/29" mail="gilles.filippini@free.fr">
// WITH_OLDLILY trick implemented to be able to coop with old LilyPond syntax
// </GFI>

#include <stdlib.h> //  esigra: needed for mkstemp when --without-libs
#include "config.h"
#include <iostream>
#include <strstream.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qregexp.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qslider.h>
#include "lilyexport.h"
#include "mainframewidget.h"
#include "resource.h"
#include "clef.h"
#include "keysig.h"
#include "timesig.h"
#include "sign.h"
#include "rest.h"
#include "voice.h"
#include "staff.h"
#include "uiconnect.h"
#include "filehandler.h"

#define MEASURES_PER_LINE 2

#define EDGE_TEXT_NOT_SET 0
#define EDGE_TEXT_TR 1
#define EDGE_TEXT_EMPTY 2

#define BODY_DEFAULT 0

extern bool lilySemicolons_; 
extern bool lilyVarTrills_;

NLilyExport::NLilyExport() :
relSyms("[<>]"), whiteSpaces_(" "), nonAlphas_("[^A-Za-z]"),
digits_("[0-9]") {
	os_ = new ostrstream(buffer_, 128);
	noStrongPizzMsg_ = false;
}

void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFrm *expWin, NMainFrameWidget *mainWidget) {
	mainWidget_ = mainWidget;
	staffList_ = stafflist;
	staffCount_ = stafflist->count();
	fileName = fname;
	exportDialog_ = expWin;
	this->doExport();
}


void NLilyExport::doExport() {
	unsigned int i, j;
	double wh;
	int k, str;
	badmeasure *bad;
	NVoice *voice_elem;
	NStaff *staff_elem;
	int voice_count;
	QString staffName, staffLabel, s;
	NClef *clef;
	bool header_written = false;

	if (fileName.isNull())
		return;

	out_.open(fileName);
	if (!out_) {
		*os_ << "error opening file " << fileName << '\0';
		KMessageBox::sorry
			(0, QString(os_->str()), kapp->makeStdCaption(i18n("???")));
		return;
	}
	out_.setf(ios::showpoint);
	lastLilyPondfile_ = fileName;
	staffarray_ = new staffdescr[staffCount_];
	noteBody_ = BODY_DEFAULT;
	for (i = 0; i < staffCount_; i++) {
		staffarray_[i].lyrics_count = 0;
		if (NResource::staffSelExport_ != 0) {
			staffarray_[i].is_selected = NResource::staffSelExport_[i];
		}
		else {
			staffarray_[i].is_selected = true;
		}
	}
	badlist_.setAutoDelete(true);
	badlist_.clear();
	drum_problem_written_ = false;
	out_ << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
	out_ << "%                                  %" << endl;
	out_ << "%   LilyPond output generated by   %" << endl;
	out_ << "%           \"noteedit\"             %" << endl;
	out_ << "%                                  %" << endl;
	out_ << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl << endl;
	if (!exportDialog_->lilyVoice->isChecked()) {
		out_ << "\\include \"paper" << exportDialog_->lilyFont->currentText() << ".ly\"" << endl << endl;
	}
	out_ << "bcr = {" << endl << "\\property Voice.NoteHead \\set #'style = #'cross" << endl << '}' << endl;
	out_ << "bdf = {" << endl << "\\property Voice.NoteHead \\set #'style = #'default" << endl << '}' << endl;
	out_ << "bcc = {" << endl << "\\property Voice.NoteHead \\set #'style = #'xcircle" << endl << '}' << endl;
	out_ << "btr = {" << endl << "\\property Voice.NoteHead \\set #'style = #'triangle" << endl << '}' << endl;


	if (!mainWidget_->scTitle_.isEmpty()) {
		if (!header_written) {
			header_written = true;
			out_ << endl << "\\header {" << endl;
		}
		out_ << "\ttitle = \"" << mainWidget_->scTitle_ << '"';
		if (lilySemicolons_) out_ << ";";
		out_ << endl;
	}
	if (!mainWidget_->scAuthor_.isEmpty()) {
		if (!header_written) {
			header_written = true;
			out_ << endl << "\\header {" << endl;
		}
		out_ << "\tcomposer = \"" << mainWidget_->scAuthor_ << '"';
		if (lilySemicolons_) out_ << ";";
		out_ << endl;
	}
	if (header_written) {
		out_ << '}' << endl << endl;
	}
	for (i = 0, staff_elem = staffList_->first(); staff_elem; staff_elem = staffList_->next(), i++) {
		if (!staffarray_[i].is_selected) continue; 
		voice_count = staff_elem->voiceCount();
		if (voice_count == 1) {
			if (staff_elem->staffName_.isNull()) {
				staffLabel.sprintf("Staff%c", i + 'A');
			}
			else {
				staffLabel = staff_elem->staffName_ + s.sprintf("%c", i + 'A');
			}
			removeExceptsFromString(&staffLabel, false);
			out_ << staffLabel;
			out_ << " = \\notes\\relative c";
			voice_elem = staff_elem->getVoiceNr(0);
			clef = voice_elem->getFirstClef();
			switch (clef->getSubType()) {
				case DRUM_BASS_CLEF:
				case DRUM_CLEF: lastLine_ = -3; str = 0; break;
				case TENOR_CLEF: lastLine_ = 3; str = 1; break;
				case ALTO_CLEF: lastLine_ = 4; str = 1; break;
				case BASS_CLEF: lastLine_ = 3; str = 0; break;
				case TREBLE_CLEF: lastLine_ = -2; str = 1; break;
			} 
			for (k = 0; k < str; k++) out_ << '\'';
			out_ << " {" << endl << '\t';
			if (exportDialog_->lilyBeam->isChecked()) {
				out_ << "\\property Voice.noAutoBeaming = ##t" << endl << '\t';
			}
			if (exportDialog_->lilyTies->isChecked()) {
				out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
			}
			if (!voice_elem->getStaff()->staffName_.isNull()) {
				out_ << "\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl << '\t';
			}
			voice_elem->prepareForWriting();
			staff_elem->actualClef_.change(clef);
			depth_ = 1;
			writeVoice(i, 0,voice_elem);
			out_ << "}" << endl;
		}
		else {
			voice_elem = staff_elem->getVoiceNr(0);
			clef = voice_elem->getFirstClef();
			for (j = 0; j < voice_count; j++) {
				voice_elem = staff_elem->getVoiceNr(j);
				if (staff_elem->staffName_.isNull()) {
					staffLabel.sprintf("Staff%cVoice%c", i + 'A', j + 'A');
				}
				else {
					staffLabel = staff_elem->staffName_ + s.sprintf("%cVoice%c", i + 'A', j + 'A');
				}
				removeExceptsFromString(&staffLabel, false);
				out_ << staffLabel;
				out_ << " = \\notes\\relative c";
				switch (clef->getSubType()) {
					case DRUM_BASS_CLEF:
					case DRUM_CLEF: lastLine_ = -3; str = 0; break;
					case TENOR_CLEF: lastLine_ = 3; str = 1; break;
					case ALTO_CLEF: lastLine_ = 4; str = 1; break;
					case BASS_CLEF: lastLine_ = 3; str = 0; break;
					case TREBLE_CLEF: lastLine_ = -2; str = 1; break;
				} 
				for (k = 0; k < str; k++) out_ << '\'';
				out_ << " {" << endl << '\t';
				if (exportDialog_->lilyBeam->isChecked()) {
					out_ << "\\property Voice.noAutoBeaming = ##t" << endl << '\t';
				}
				if (exportDialog_->lilyTies->isChecked()) {
					out_ << "\\property Thread.sparseTies = ##t" << endl << '\t';
				}
				voice_elem->prepareForWriting();
				staff_elem->actualClef_.change(clef);
				depth_ = 1;
				writeVoice(i, j, voice_elem);
				if (j < 0) out_ << endl;
				out_ << "}" << endl;
			}
			if (staff_elem->staffName_.isNull()) {
				staffLabel.sprintf("Staff%c", i + 'A');
			}
			else {
				staffLabel = staff_elem->staffName_ + s.sprintf("%c", i + 'A');
			}
			removeExceptsFromString(&staffLabel, false);
			out_ << staffLabel << " = \\simultaneous {" << endl;
			if (!staff_elem->staffName_.isNull()) {
				out_ << "\t\\property Staff.instrument = #\"" << staff_elem->staffName_ << '"' << endl;
			}
			for (j = 0, voice_elem = staff_elem->getVoiceNr(0); j < voice_count; j++) {
				if (staff_elem->staffName_.isNull()) {
					staffLabel.sprintf("Staff%cVoice%c", i + 'A', j + 'A');
				}
				else {
					staffLabel = staff_elem->staffName_ + s.sprintf("%cVoice%c", i + 'A', j + 'A');
				}
				removeExceptsFromString(&staffLabel, false);
				out_ << "\t\\context Voice=\"" << staffLabel << "\" " <<  "\\" << staffLabel << endl;
			}
			out_ << "}" << endl;
		}
		if (staff_elem->staffName_.isNull()) {
			staffLabel.sprintf("Staff%c", i + 'A');
		}
		else {
			staffLabel = staff_elem->staffName_ + s.sprintf("%c", i + 'A');
		}
		removeExceptsFromString(&staffLabel, false);
		if (staffarray_[i].lyrics_count) {
			out_ << staffLabel << "Text = \\lyrics ";
			writeLyrics(i, voice_elem);
		}
	}
	if (!exportDialog_->lilyVoice->isChecked()) {
		out_ << "\\score {" << endl;
		out_ << "\t\\simultaneous {" << endl;
		out_ << "\t\t\\property Score.skipBars = ##t" << endl;
	
		for (i = 0, staff_elem = staffList_->first(); staff_elem; staff_elem = staffList_->next(), i++) {
			if (!staffarray_[i].is_selected) continue;
	
			if (staff_elem->staffName_.isNull()) {
				staffLabel.sprintf("Staff%c", i + 'A');
				staffName.sprintf("Staff%c", i + 'A');
			}
			else {
				staffLabel = staff_elem->staffName_ + s.sprintf("%c", i + 'A');
				staffName = staff_elem->staffName_;
			}
			removeExceptsFromString(&staffLabel, false);

			if (staffarray_[i].lyrics_count) {
				out_ << "\t\t\\addlyrics " << endl << '\t';
			}
	                out_ << "\t\t\\context Staff=\"" << staffName << "\" ";
			out_ << '\\' << staffLabel << endl;
			if (staffarray_[i].lyrics_count) {
				out_ << "\t\t\t\\context Lyrics=\"" << staffName << "Text\" \\" << staffLabel << "Text" << endl;
			}
		}
	
	
		out_ << "\t}" << endl;
	        out_ << "\t\\paper {" << endl;
		if (exportDialog_->lilyLand->isChecked()) {
			out_ << "\t\torientation = \"landscape\"" << endl;
			if (lilySemicolons_) out_ << ";";
		}
		if (sscanf(exportDialog_->lilyWidth->text(), "%lf", &wh) != 1) {
			wh = 250.0;
		}
	        out_ << "\t\tlinewidth = " << wh << " \\mm";
		if (lilySemicolons_) out_ << ";";
		out_ << endl;
		if (sscanf(exportDialog_->lilyHeight->text(), "%lf", &wh) != 1) {
			wh = 160.0;
		}
	        out_ << "\t\ttextheight = " << wh << " \\mm";
		if (lilySemicolons_) out_ << ";";
		out_ << endl;
	        out_ << "\t}" << endl;
		out_ << "}" << endl;
	}
	out_.close();
	delete [] staffarray_;
	if (!badlist_.isEmpty()) {
		QString output;
		output = i18n
			("Noteedit has exported the score to LilyPond but there are some\n"
			 "problems which will probably prevent successful LilyPond output.\n");
		output += i18n("-----------------------------------------------------\n");
		for (bad = badlist_.first(); bad; bad = badlist_.next()) {
			switch (bad->kind) {
				case ERR_NOTE_128:
				output += i18n
				( "Staff %1, measure %2: 128th in mesaure; minimum is 64th\n").arg(bad->track).arg(bad->measure);
					break;
				case ERR_IRREGULAER:
				output += i18n
				( "Staff %1 has irregular keysig. This cannot be expressed in LilyPond\n").arg(bad->track);
					break;
				case ERR_NOTE_COUNT:
				output += i18n
				( "Staff %1, measure %2: has %3 128th, should be: %4\n").
					arg(bad->track).arg(bad->measure).arg(bad->realcount).arg(bad->shouldbe);
					break;
				case ERR_VAR_TRILLS:
				output += i18n
				( "Staff %1, measure %2: LilyPond less than 1.5.2 cannot deal with variable length trills (wavelines)\n").
				arg(bad->track).arg(bad->measure);
					break;
				case ERR_NOTE_MIXED_TIES:
				output += i18n 
				( "Staff %1, measure %2: Only a subset of notes in a chord has ties!\n").arg(bad->track).arg(bad->measure);
					break;
				case ERR_DRUM_STAFF:
					output += i18n
						("Staff %1 contains drum staff. Note! The LilyPond export is not quite correct!\n")
						.arg(bad->track);
					break;
			}
		}
		NResource::exportWarning_->setOutput(i18n ("LilyPond produced. But there are some problems."), &output);
		NResource::exportWarning_->show();
	}
}



void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
	NMusElement *elem, *specialElem;
	char *hiddenRepeat = "\\repeat \"volta\" 2 {";
	int total = 0, i;
	bool first;
	bool AfterFirstBar = false;
	char *pendingVolSig = 0;
	int acr;
	int saveLine;
	int LineBeforeSPE1;
	bool lastElemIsBar = false;
	bool prevElemIsBar = false;
	bool repeatOpenSeen = false;
	bool specialEnding2Seen = false;
	bool intriplet = false, inbeam = false;
	long lastBarPos = 0;
	long currentPos;
	long PosOfHiddenRepeat = -1;
	bool repeatOpenAtBeginMissing = false;
	int length;
	int part;
	NClef *clef;
	NKeySig *key;
	NTimeSig *timesig;
	NNote *note;
	NChord *chord;
	NSign *sign;
	int counter = 0;
	int measure_counter = 0;
	int kind, count;
	NStaff *actual_staff;
	int dynEndPos = 0;
	char *lastDynSym = 0;
	bool textpropset = false;
	int edgetext = EDGE_TEXT_NOT_SET;
	int trillendpos = 0;
	int trilledNotes = 0;
	int c4Line;

	countof128th_ = 128;
	currentNumerator_ = currentDenominator_ = 4;
	barNr_ = 1;
	badmeasure *bad;
	

	lastLength_ = 1000;
	lastDotted_ = 0;
	actualStemPolicy_ = STEM_UNSET;
	actual_staff = voi->getStaff();
	actual_staff->actualKeysig_.resetAtBar();
	elem = voi->getCurrentPosition();
	if (elem == 0) return;
	if (!voi->isFirstVoice()) {
		actual_staff->resetSpecialElement();
	}
	do {
		lastElemIsBar = false;
		if (!voi->isFirstVoice()) {
			while (specialElem = actual_staff->checkSpecialElement(elem->getXpos())) {
				switch (specialElem->getType()) {
					case T_KEYSIG: actual_staff->actualKeysig_.change((NKeySig *) specialElem); break;
					case T_CLEF: c4Line = actual_staff->actualClef_.lineOfC4();
						     actual_staff->actualClef_.change((NClef *) specialElem);
						     lastLine_ += actual_staff->actualClef_.lineOfC4() - c4Line;
						     break; 
				}
			}
		}
		switch (elem->getType()) {
			case T_CHORD: if (PosOfHiddenRepeat < 0) {
					if (acr = voi->determineAnacrusis()) {
						out_ << "\\partial " << (128 / acr);
						if (lilySemicolons_) out_ << ";";
						out_ << endl << '\t';
					}
					PosOfHiddenRepeat = out_.tellp();
					out_ << hiddenRepeat << endl << '\t';
				     }
				     prevElemIsBar = false;
				     part = elem->getSubType(); 
				     total += (elem->status_ & STAT_TRIPLET) ? 2 * part / 3 : part;
				     switch (elem->status_ & DOT_MASK) {
					case 1: total += part / 2; break;
					case 2: total += 3 * part / 4; break;
				     }
				     if ((elem->status_ & STAT_TRIPLET) && !intriplet) {
					intriplet = true;
					out_ << "\\times 2/3 { ";
				     }
				     if (total > 3*countof128th_ && voi->isFirstVoice()) {
					total = 0;
					bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
					badlist_.append(bad);
				     }
				     chord = (NChord *) elem;
				     if (chord->countOfLyricsLines() > staffarray_[staff_nr].lyrics_count) {
					staffarray_[staff_nr].lyrics_count = chord->countOfLyricsLines();
				     }
				     if (chord->getSubType() < 3*2) {
					part = 3*2;	
					bad = new badmeasure(ERR_NOTE_128, (staff_nr + 1), barNr_, total / 3, countof128th_);
					badlist_.append(bad);
				     }
				     if (chordHasMixedTies(elem->getNoteList())) {
					bad = new badmeasure(ERR_NOTE_MIXED_TIES, (staff_nr + 1), barNr_, total / 3, countof128th_);
					badlist_.append(bad);
				     }
				     if (chord->trill_) {
					trilledNotes = voi->findNoteCountTillTrillEnd(chord);
				        if (lilyVarTrills_) {
						if (!textpropset) {
							textpropset = true;
							out_ << endl;
							tabsOut();
							out_ << "\\property Voice.TextSpanner \\set #'type = #'trill" << endl;
						}
						if (chord->trill_ > 0 && edgetext != EDGE_TEXT_TR) {
							edgetext = EDGE_TEXT_TR;
							out_ << endl;
							tabsOut();
							out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'((line (music \"scripts-trill\") \" \") . \"\")" << endl;
							tabsOut();
						}
						else if (chord->trill_ < 0 && edgetext != EDGE_TEXT_EMPTY) {
							edgetext = EDGE_TEXT_EMPTY;
							out_ << endl;
							tabsOut();
							out_ << "\\property Voice.TextSpanner \\set #'edge-text = #'(\"\")" << endl;
							tabsOut();
						}
					}
					else {
						trilledNotes = 0;
						bad = new badmeasure(ERR_VAR_TRILLS, (staff_nr + 1), barNr_, total / 3, countof128th_);
						badlist_.append(bad);
					}
				     }
				     if (hasContraryStems(elem->getNoteList()) || exportDialog_->lilyStem->isChecked()) {
					if (chord->status_ & STAT_STEM_UP) {
						if (actualStemPolicy_ != STEM_DIR_UP) {
#ifndef WITH_OLDLILY
							out_ << "\\stemUp ";
#else
							out_ << "\\stemup ";
#endif	// WITH_OLDLILY
							actualStemPolicy_ = STEM_DIR_UP;
						}
					}
					else {
						if (actualStemPolicy_ != STEM_DIR_DOWN) {
#ifndef WITH_OLDLILY
							out_ << "\\stemDown ";
#else
							out_ << "\\stemdown ";
#endif	// WITH_OLDLILY
							actualStemPolicy_ = STEM_DIR_DOWN;
						}
					}
				     }
				     else if (actualStemPolicy_ != STEM_UNSET) {
#ifndef WITH_OLDLILY
					out_ << "\\stemBoth ";
#else
					out_ << "\\stemboth ";
#endif	// WITH_OLDLILY
					actualStemPolicy_ = STEM_UNSET;
				     }
				     if (exportDialog_->lilyBeam->isChecked()) {
				     	if ((elem->status_ & STAT_BEAMED) && !inbeam) {
						out_ << "[ "; inbeam = true;
				     	}
				     }
				     if (elem->getNoteList()->count() > 1) out_ << "< ";
				     first = true;
			  	     for (note = elem->getNoteList()->first(); note; note = elem->getNoteList()->next()) {
					     if (first) {
					        if (chord->status_ & STAT_PART_OF_SLUR) {
							out_ << ") ";
					        }
					     }
					     if (!drum_problem_written_ && (note->status & BODY_MASK)) {
						drum_problem_written_ = true;
						bad = new badmeasure(ERR_DRUM_STAFF, staff_nr +1, 3 /* dummy */, total / 3, countof128th_);
						badlist_.append(bad);
					     }
					     if (noteBody_ != (note->status & BODY_MASK)) {
						switch (noteBody_ = (note->status & BODY_MASK)) {
							case STAT_BODY_CROSS:
							case STAT_BODY_CROSS2:
								out_ << "\\bcr "; break;
							case STAT_BODY_CIRCLE_CROSS:
							case STAT_BODY_RECT:
								out_ << "\\bcc "; break;
							case STAT_BODY_TRIA:
								 out_ << "\\btr "; break;
							default:
								 out_ << "\\bdf "; break;
						}
					      }
					     pitchOut(&(actual_staff->actualKeysig_), note, &(actual_staff->actualClef_), true);
					     length = 3*128 / part;
					     if (first) {
						first = false;
					        if (length != lastLength_ || lastDotted_ != (elem->status_ & DOT_MASK)) {
						   if (part == 3*256) {
							out_ << "\\breve";
						   }
						   else {
						   	out_ << length;
						   }
						   lastLength_ = length;
						   switch (elem->status_ & DOT_MASK) {
							case 1:	out_ << "."; break;
							case 2:	out_ << ".."; break;
						   }
						   lastDotted_ = elem->status_ & DOT_MASK;
					        }
						if (elem->status_ & STAT_STACC) {
							out_ << "-.";
					        }
						if (elem->status_ & STAT_SFORZ) {
							out_ << "-^";
						}
						if (elem->status_ & STAT_PORTA) {
							out_ << "--";
						}
						if (elem->status_ & STAT_STPIZ) {
							if (!noStrongPizzMsg_)
								KMessageBox::sorry
									(0,
									 i18n("Sorry, strong Pizzicato is not available under LilyPoint. Take musixtex, to enjoy this feature."),
									 kapp->makeStdCaption(i18n("???"))
									);
							noStrongPizzMsg_ = true;
						}
						if (elem->status_ & STAT_SFZND) {
							out_ << "->";
						}
						if (elem->status_ & STAT_FERMT) {
							out_ << "\\fermata";
						}
						if (chord->trill_ > 0 && trilledNotes == 0) {
							out_ << "\\trill ";
						}
						else if (chord->trill_ < 0 && trilledNotes == 0) {
							out_ << "\\prall ";
						}
						if (trillendpos != 0 && chord->getBbox()->right() > trillendpos) {
							trillendpos = 0;
							out_ <<  " \\spanrequest \\stop \"text\" ";
						}
						if (chord->trill_ != 0 && trilledNotes > 0) {
							trillendpos = chord->getTrillEnd();
							out_ << " \\spanrequest \\start \"text\" ";
						}
						if (chord->dynamic_) {
							out_ << (chord->dynamicAlign_ ? " \\cr  " : " \\decr ");
							dynEndPos = chord->getDynamicEnd();
							lastDynSym = (char *) (chord->dynamicAlign_ ? " \\rc " : " \\rced ");
						}
						else if (lastDynSym && chord->getBbox()->right() >  dynEndPos) {
							out_ << lastDynSym;
							lastDynSym = 0;
							dynEndPos = 0;
						}
					        if (pendingVolSig) {
						   switch (exportDialog_->lilyVol->currentItem()) {
							case 1: out_ << '^' << pendingVolSig; break;
							case 2: out_  << pendingVolSig; break;
						   }
						   pendingVolSig = 0;
						}
						saveLine = lastLine_;
						if (chord->status_ & STAT_SLURED) {
						   out_ << " ( ";
						}
					     }
				     	     out_ << ' ';
				     }
				     if (elem->getNoteList()->count() > 1) out_ << "> ";
				     if (exportDialog_->lilyBeam->isChecked()) {
				     	if (elem->lastBeamed()) {
						out_ << "] "; inbeam = false;
				     	}
				     }
				     if (hasATie(elem->getNoteList())) {
					out_ << "~ ";
				     }
				     lastLine_ = saveLine;
				     actual_staff->actualKeysig_.makeTmpActual();
				     if (elem->status_ & STAT_LAST_TRIPLET) {
					intriplet = false;
					out_ << " } ";
				     }
				     break;
			case T_REST: if (PosOfHiddenRepeat < 0) {
					if (acr = voi->determineAnacrusis()) {
						out_ << "\\partial " << (128 / acr);
						if (lilySemicolons_) out_ << ";";
						out_ << endl << '\t';
					}
					PosOfHiddenRepeat = out_.tellp();
					out_ << hiddenRepeat << endl << '\t';
				     }
				     prevElemIsBar = false;
				     part = elem->getSubType(); 
				     if (part == MULTIREST) {
					total = 3*countof128th_;
				     }
				     else {
				     	total += (elem->status_ & STAT_TRIPLET) ? 2 * part / 3 : part;
				     }
				     switch (elem->status_ & DOT_MASK) {
					case 1: total += part / 2; break;
					case 2: total += 3 * part / 4; break;
				     }
				     if ((elem->status_ & STAT_TRIPLET) && !intriplet) {
					intriplet = true;
					out_ << "\\times 2/3 { ";
				     }
				     if (total > 3*countof128th_ && voi->isFirstVoice()) {
					total = 0;
					bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
					badlist_.append(bad);
				     }
				     if (part == MULTIREST) {
					lastDotted_ = lastLength_ = 1111; /* invalidate */
					out_ << 'R' << currentDenominator_ << '*' <<
						(currentNumerator_ * ((NRest *)elem)->getMultiRestLength());
				     }
				     else {
				     	out_ << ((elem->status_ & STAT_HIDDEN) ? "s" : "r");
				     	length = 3*128 / part;
				     	if (length != lastLength_  || lastDotted_ != (elem->status_ & DOT_MASK)) {
						out_ << length;
						lastLength_ = length;
				     		switch (elem->status_ & DOT_MASK) {
							case 1:	out_ << "."; break;
							case 2:	out_ << ".."; break;
				     		}
						lastDotted_ = elem->status_ & DOT_MASK;
				     	}
				      }
				     if (pendingVolSig) {
					switch (exportDialog_->lilyVol->currentItem()) {
						case 1: out_ << '^' << pendingVolSig; break;
						case 2: out_ << pendingVolSig; break;
					}
				        pendingVolSig = 0;
				     }
				     out_ << ' ';
				     if (elem->status_ & STAT_LAST_TRIPLET) {
					intriplet = false;
					out_ << " } ";
				     }
				     break;
			case T_KEYSIG: key = (NKeySig *) elem;
				     prevElemIsBar = false;
				     if (key->isRegular(&kind, &count)) {
					if (count || barNr_ > 1) {
						out_ << "\\key " << LilyPondKeyName(kind, count) << " \\major";
						if (lilySemicolons_) out_ << ";";
						out_ << endl; tabsOut();
					}
				     }
				     else {
					bad = new badmeasure(ERR_IRREGULAER, (staff_nr + 1), barNr_, total / 3, countof128th_);
					badlist_.append(bad);
				     }
				     break;
			case T_TIMESIG: timesig = (NTimeSig *) elem;
					if (prevElemIsBar) {
						out_.seekp(lastBarPos);
					}
					countof128th_ = timesig->numOf128th();
					currentNumerator_ = timesig->getNumerator();
					currentDenominator_ = timesig->getDenominator();
					out_ << "\\time " << timesig->getNumerator() << '/'
					                  << timesig->getDenominator();
					    		  if (lilySemicolons_) out_ << ";";
					out_ << ' ';
					if (prevElemIsBar) {
						lastBarPos = out_.tellp();
						out_ << "| ";
						prevElemIsBar = true;
					}
					out_ << endl; tabsOut();
				     break;
			case T_CLEF: clef = (NClef *) elem;
				     prevElemIsBar = false;
			             out_ << "\\clef ";
				     switch (clef->getSubType()) {
					 case DRUM_CLEF:  out_ << "percussion"; if (lilySemicolons_) out_ << ";"; break;
				         case TREBLE_CLEF: out_ << "violin"; if (lilySemicolons_) out_ << ";"; break;
					 case TENOR_CLEF: out_ << "tenor"; if (lilySemicolons_) out_ << ";"; break;
					 case ALTO_CLEF: out_ << "alto"; if (lilySemicolons_) out_ << ";"; break;
					 case DRUM_BASS_CLEF: out_ << "percussion"; if (lilySemicolons_) out_ << ";"; break;
					 case BASS_CLEF: out_ << "bass"; if (lilySemicolons_) out_ << ";"; break;
				     }
				     out_ << endl; tabsOut();
				     c4Line = actual_staff->actualClef_.lineOfC4();
				     actual_staff->actualClef_.change(clef);
				     lastLine_ += actual_staff->actualClef_.lineOfC4() - c4Line;
				     break;
			case T_SIGN: switch (elem->getSubType ()) {
					case SIMPLE_BAR: 
						if (AfterFirstBar && total != 3*countof128th_ && voi->isFirstVoice()) {
							bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
							badlist_.append(bad);
						}
						total = 0;
						if (specialEnding2Seen) {
#ifndef WITH_OLDLILY
							lastLine_ = LineBeforeSPE1;
#endif	// WITH_OLDLILY
							specialEnding2Seen = false;
							counter = 0;
							if (lastElemIsBar) {
								out_.seekp(lastBarPos);
								out_ << "| " << endl;
							}
							depth_--; out_ << endl; tabsOut();
							out_ << "}" << endl; depth_--; tabsOut();
							out_ << "}" << endl; tabsOut();
							lastLength_ = 1000; // invalidate
							break;
						}
						lastElemIsBar = true;
						lastBarPos = out_.tellp();
						out_ << "| ";
						prevElemIsBar = true;
						if (exportDialog_->lilyMeasure->isChecked()) {
							if (((++measure_counter) % exportDialog_->lilyMeasureVal->value()) == 0) {
								out_ << "\\break ";
							 }
						}
						if (((++counter) % MEASURES_PER_LINE) == 0) {
							out_ << "  % " << ((NSign *) elem)->getBarNr();
							out_ << endl; tabsOut();
						}
						barNr_ = ((NSign *) elem)->getBarNr();
						AfterFirstBar = true;
						break;
					case REPEAT_OPEN:
				     		prevElemIsBar = false;
						if (AfterFirstBar && total != 3*countof128th_ && voi->isFirstVoice()) {
							bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
							badlist_.append(bad);
						}
						total = 0;
						if (specialEnding2Seen) {
#ifndef WITH_OLDLILY
							lastLine_ = LineBeforeSPE1;
#endif	// WITH_OLDLILY
							specialEnding2Seen = false;
							counter = 0;
							if (lastElemIsBar) {
								out_.seekp(lastBarPos);
								out_ << "| " << endl;
							}
							depth_--; out_ << endl; tabsOut();
							out_ << "}" << endl; depth_--; tabsOut();
							out_ << "}" << endl; tabsOut();
							lastLength_ = 1000; // invalidate
						}
						repeatOpenSeen = true;
						specialEnding2Seen = false;
						++measure_counter; ++counter;
						out_ << endl; tabsOut(); counter = 0;
						out_ << "\\repeat \"volta\" 2 {" << endl; depth_++; tabsOut();
						barNr_ = ((NSign *) elem)->getBarNr();
						AfterFirstBar = true;
						lastLength_ = 1000; // invalidate
						break;
					case REPEAT_CLOSE:
				     		prevElemIsBar = false;
						if (AfterFirstBar && total != 3*countof128th_ && voi->isFirstVoice()) {
							bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
							badlist_.append(bad);
						}
						total = 0;
						if (!repeatOpenSeen) {
							repeatOpenAtBeginMissing = true;
						}
						repeatOpenSeen = true;
						if (lastElemIsBar) {
							out_.seekp(lastBarPos);
							out_ << "| " << endl;
						}
						if (exportDialog_->lilyMeasure->isChecked()) {
							if (((++measure_counter) % exportDialog_->lilyMeasureVal->value()) == 0) {
								out_ << "\\break ";
							 }
						}
						if (repeatOpenSeen) depth_--; out_ << endl; tabsOut();
						out_ << "}" << endl; tabsOut();
						counter = 0;
						barNr_ = ((NSign *) elem)->getBarNr();
						AfterFirstBar = true;
						lastLength_ = 1000; // invalidate
						break;
				
					case REPEAT_OPEN_CLOSE:
				     		prevElemIsBar = false;
						if (AfterFirstBar && total != 3*countof128th_ && voi->isFirstVoice()) {
							bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
							badlist_.append(bad);
						}
						total = 0;
						if (!repeatOpenSeen) {
							repeatOpenAtBeginMissing = true;
						}
						if (lastElemIsBar) {
							out_.seekp(lastBarPos);
							out_ << "| " << endl;
						}
						if (repeatOpenSeen) depth_--; out_ << endl; tabsOut();
						out_ << "}" << endl; tabsOut();
						counter = 0;


						if (!repeatOpenSeen) {
							repeatOpenAtBeginMissing = true;
						}
						specialEnding2Seen = false;
						++measure_counter; ++counter;
						out_ << endl; tabsOut(); counter = 0;
						out_ << "\\repeat \"volta\" 2 {" << endl; depth_++; tabsOut();
						barNr_ = ((NSign *) elem)->getBarNr();
						AfterFirstBar = true;
						lastLength_ = 1000; // invalidate
						if (exportDialog_->lilyMeasure->isChecked()) {
							if (((++measure_counter) % exportDialog_->lilyMeasureVal->value()) == 0) {
								out_ << "\\break ";
							 }
						}
						break;
					case SPECIAL_ENDING1: 
				     		prevElemIsBar = false;
						if (lastElemIsBar) {
							out_.seekp(lastBarPos);
							out_ << "| " << endl;
						}
						depth_--; out_ << endl; tabsOut();
						out_ << "}" << endl; tabsOut();
						out_ << "\\alternative {" << endl; depth_++; tabsOut();
						out_ << "{" << endl; depth_++; tabsOut(); counter = 0;
						LineBeforeSPE1 = lastLine_;
						lastLength_ = 1000; // invalidate
						break;
					case SPECIAL_ENDING2:
				     		prevElemIsBar = false;
						counter = 0;
						out_ << "{" << endl; depth_++; tabsOut(); 
						specialEnding2Seen = true;
						lastLength_ = 1000; // invalidate
#ifndef WITH_OLDLILY
						lastLine_ = LineBeforeSPE1;
#endif	// WITH_OLDLILY
						break;
#ifdef XXX
					case TEMPO_SIGNATURE:
				     		prevElemIsBar = false;
						if (!loop1) break;
						sign = (NSign *) elem;
						out_ << "midi all: 0 \"tempo=" << sign->getTempo() <<
							 "\"";
							 if (lilySemicolons_) out_ << ";";
						out_ << "rom above all: 1 \"( \\(sm4n) = " << 
							sign->getTempo() << " )\"";
							if (lilySemicolons_) out_ << ";";
						break;
#endif
					case VOLUME_SIG:
				     		prevElemIsBar = false;
						sign = (NSign *) elem;
						switch(sign->getVolType()) {
							case V_PPPIANO : pendingVolSig = "\\ppp "; break;
							case V_PPIANO  : pendingVolSig = "\\pp "; break;
							case V_PIANO   : pendingVolSig = "\\p "; break;
							case V_FORTE   : pendingVolSig = "\\f "; break;
							case V_FFORTE  : pendingVolSig = "\\ff "; break;
							case V_FFFORTE : pendingVolSig = "\\fff "; break;
							default        : pendingVolSig = "\\mf "; break;
						}
						break;
#ifdef XXX
					case PROGRAM_CHANGE:
				     		prevElemIsBar = false;
						sign = (NSign *) elem;
						out_ << "midi " << staff_nr << " 1: 0 \"program=" << sign->getProgram() << "\"";
						if (lilySemicolons_) out_ << ";";
						break;
#endif
				     }
				     break;
		}
		elem = voi->getNextPosition();
	}
	while (elem);
	if (voice_nr == 0 && total > 3*countof128th_ && voi->isFirstVoice()) {
		bad = new badmeasure(ERR_NOTE_COUNT, (staff_nr + 1), barNr_, total / 3, countof128th_);
		badlist_.append(bad);
	}
	if (lastElemIsBar) {
		out_.seekp(lastBarPos);
		out_ << endl;
		tabsOut();
		out_ << "\\bar \"|.\"";
		if (lilySemicolons_) out_ << ";";
		out_ << endl;
	}
	if (!repeatOpenAtBeginMissing) {
		currentPos = out_.tellp();
		out_.seekp(PosOfHiddenRepeat);
		for (i = 0; i < strlen(hiddenRepeat); i++) out_ << ' ';
		out_.seekp(currentPos);
	}
	if (voice_nr > 0) out_ << endl;
}

void NLilyExport::writeLyrics(int voice_nr, NVoice *voi) {
	NMusElement *elem;
	NChord *chord;
	QString *ps, s;
	int lyline;
	int counter;

	if (staffarray_[voice_nr].lyrics_count > 1) {
		out_ << "{" << endl << "\t\\simultaneous {" << endl << "\t\t";
		depth_ = 2;
	}
	else {
		out_ << "{" << endl << '\t';
		depth_ = 1;
	}
	
	for (lyline = 0; lyline < staffarray_[voice_nr].lyrics_count; lyline++) {
		counter = 0;
		voi->prepareForWriting();
		out_ << "{" << endl; depth_++; tabsOut();
		elem = voi->getCurrentPosition();
		while (elem) {
			switch (elem->getType()) {
			case T_SIGN:
				if (elem->getSubType() & BAR_SYMS) {
					if (((++counter) % MEASURES_PER_LINE) == 0) {
						out_ << endl; tabsOut();
					}
				}
				break;
			case T_CHORD:
				chord = (NChord *) elem;
				ps = chord->getLyrics(lyline);
				if (!ps) {
					out_ << " _ ";
				}
				else {
					s = QString(*ps);
					s.replace(relSyms, "");
					s.replace(whiteSpaces_, "_");
					removeExceptsFromString(&s, true);
					out_ << s << ' ';
				}
				break;
			}
			elem = voi->getNextPosition();
		}
		depth_--;
		out_ << '}' << endl; tabsOut();
	}
	if (staffarray_[voice_nr].lyrics_count > 1) {
		out_ << '}' << endl;
	}
	out_ << '}' << endl;
}


void NLilyExport::removeExceptsFromString(QString *str, bool onlyDigits) {
	int l = str->length();
	int digit;
	QChar c, c2;
	while ((l = str->find(digits_)) >= 0) {
		c = str->at(l);
		digit = c.digitValue ();
		c2 = QChar('A' + digit);
		str->replace(l, 1, &c2, 1);
	}
	if (onlyDigits) return;
	str->replace(nonAlphas_, "X");
		
}
		
char *NLilyExport::LilyPondKeyName(int kind, int count) {
	char *err = "LilyPondKeyName: internal error";
	if (kind == STAT_CROSS) {
		switch (count) {
			case 0: return "c";
			case 1: return "g";
			case 2: return "d";
			case 3: return "a";
			case 4: return "e";
			case 5: return "b";
			case 6: return "fis";
			default: NResource::abort(err, 1);
		}
	}
	switch (count) {
		case 0: return "c";
		case 1: return "f";
		case 2: return "bes";
		case 3: return "es";
		case 4: return "as";
		case 5: return "des";
		case 6: return "ges";
		default: NResource::abort(err, 2);
	}
	return "";
}

void NLilyExport::pitchOut( NKeySig *ksig, const NNote *note, NClef *ac_clef, bool with_tie) {
	int octave;
	int linediff;
	char notename;

	linediff = note->line - lastLine_;
	notename = ac_clef->line2Name(note->line, &octave, true);
	out_ << notename;
	switch (note->offs) {
		case 1: out_ << "is"; break;
		case -1: if (notename == 'e' || notename == 'a')
				out_ << 's';
			     else 
				out_ << "es";
			 break;
		case 2: out_ << "isis"; break;
		case -2: if (notename == 'e' || notename == 'a')
				out_ << "ses";
			     else 
				out_ << "eses";
			 break;
	}
	while (linediff > 3)
    	{
        	out_ << '\'';
          	linediff -= 7;
	}
	while (linediff < -3)
	{
        	out_ << ',';
        	linediff += 7;
	}
	if (note->status & STAT_FORCE) {
		out_ << '!';
	}
	lastLine_ = note->line;
}

bool NLilyExport::hasATie(QList<NNote> *notelist) {
	NNote *note;
	for (note = notelist->first(); note; note = notelist->next()) {
		if (note->status & STAT_TIED) return true;
	}
	return false;
}

bool NLilyExport::hasContraryStems(QList<NNote> *notelist) {
	int stemdir = STEM_UNSET;
	NNote *note;
	for (note = notelist->first(); note; note = notelist->next()) {
		if (note->line > 4 && stemdir == STEM_DIR_UP) return true;
		if (note->line <= 4 && stemdir == STEM_DIR_DOWN) return true;
		stemdir = note->line > 4 ? STEM_DIR_DOWN : STEM_DIR_UP;
	}
	return false;
}

bool NLilyExport::chordHasMixedTies(QList<NNote> *notelist) {
	bool hasTie = false;
	NNote *note;
	for (note = notelist->first(); note; note = notelist->next()) {
		if (note->status & STAT_TIED) {hasTie = true; continue;}
		if (hasTie) return true;
	}
	return false;
}

void NLilyExport::tabsOut() {
	int i;
	for (i = 0; i < depth_; i++) out_ << '\t';
}
