/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#include <istream.h>
#include <qapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kapp.h>
#include <klocale.h>
#include <kstddirs.h>
#if KDE_VERSION >= 220
#include <ktip.h>
#endif
#include "mainframewidget.h"
#include "resource.h"
#include "midimapper.h"
#include "uiconnect.h"
#include "noteedit_part.h"

static KCmdLineOptions options[] =
{
 { "+file(s)",          I18N_NOOP("Files to load"), 0 },
 { "export-lilypond",   I18N_NOOP("export file to LilyPond"), 0 },
 { "export-musixtex",   I18N_NOOP("export file to MusiXTeX"), 0 },
 { "noalsa",		I18N_NOOP("Do not create an ALSA midi scheduler"), 0 },
 { "nooss",		I18N_NOOP("Do not create an OSS midi scheduler"), 0 },
 { 0,0,0 }
};

int main( int argc, char **argv )
{
    QString lastFile;
    bool doExportLilyPond;
    bool doExportMusixTeX;
    bool doCreateAlsa;
    bool doCreateOSS;
    KCmdLineArgs::init(argc, argv, NoteeditFactory::aboutData());
    KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
    KURL u;
    KApplication a( argc, argv );
    NResource *nr = new NResource();
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
    doExportLilyPond = args->isSet("export-lilypond");
    doExportMusixTeX = args->isSet("export-musixtex");
    doCreateAlsa = args->isSet("alsa");
    doCreateOSS = args->isSet("oss");
    if (doExportLilyPond && doExportMusixTeX) {
	cerr << "you cannot specify both \"--export-musixtex\" AND \"--export-lilypond\"" << endl;
	exit(-1);
    }
    if (!doCreateAlsa && !doCreateOSS) {
	cerr << "you cannot forbid both: ALSA midi schduler and OSS midi schduler" << endl;
	doCreateAlsa = doCreateOSS = true;
    }
    if (!doCreateAlsa) NResource::schedulerRequest_ &= (~ALSA_SCHEDULER_REQUESTED);
    if (!doCreateOSS) NResource::schedulerRequest_ &= (~OSS_SCHEDULER_REQUESTED);
    NResource::mapper_ = new NMidiMapper();
    NMainWindow * mainWidget = new NMainWindow;
    switch (args->count()) {
    case 0: if (NResource::startupLoadLastScore) {
		kapp->config()->setGroup("RecentFiles");
		lastFile = kapp->config()->readEntry("File1");
		KURL u(lastFile);
		if (u.isValid() ) {
			mainWidget->mainFrameWidget()->loadFile (u.path() );
		}
	    }
	    if (doExportLilyPond) {
		cerr << "\"--export-lilypond\" ignored: file name expected" << endl;
	    }
	    if (doExportMusixTeX) {
		cerr << "\"--export-musixtex\" ignored: file name expected" << endl;
	    }
	    break;
    case 1: u = args->url(0);
	    if ( u.isLocalFile() ) {
		mainWidget->mainFrameWidget()->loadFile( u.path() );
	    	if (doExportMusixTeX) {
			NResource::commandLine_ = true;
			mainWidget->mainFrameWidget()->exportMusixTeXImm();
			exit(0);
		}
	    	if (doExportLilyPond) {
			NResource::commandLine_ = true;
			mainWidget->mainFrameWidget()->exportLilyPondImm();
			exit(0);
		}
		
	    }
	    if (doExportLilyPond) {
		cerr << "\"--export-lilypond\" ignored: file not found" << endl;
	    }
	    if (doExportMusixTeX) {
		cerr << "\"--export-musixtex\" ignored: file not found" << endl;
	    }
	    break;
    default: KCmdLineArgs::usage(); break;
    }
    args->clear(); // Free up memory.
    mainWidget->setGeometry( 0, 0, START_WIDTH, START_HEIGHT );
    //a.setMainWidget( mainWidget );
    mainWidget->show();
#if KDE_VERSION >= 220
	KTipDialog::showTip(locate("data", "noteedit/tips"));
#else
	kapp->config()->setGroup("TipOfDay");
	nr->loadTips(locate("data", "noteedit/tips"));
	if (kapp->config()->readBoolEntry("RunOnStart", true))	tipFrm( 0, NResource::tipNo_ );
#endif
    int res = a.exec();
    //delete nr;
    return res;
}
