/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#include <iostream>
#include <stdlib.h> //  esigra: needed for mkstemp when --without-libs
#include <strstream.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qregexp.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qslider.h>
#include "musixtex.h"
#include "mainframewidget.h"
#include "resource.h"
#include "clef.h"
#include "keysig.h"
#include "timesig.h"
#include "sign.h"
#include "rest.h"
#include "voice.h"
#include "staff.h"
#include "uiconnect.h"
#include "chord.h"

#define TEX_FAC 0.08

NMusiXTeX::NMusiXTeX() {
	shiftes_notes_.setAutoDelete(false);
	non_shifted_notes_.setAutoDelete(false);
}


void NMusiXTeX::exportStaffs(QString filen, QList<NStaff> *stafflist, exportFrm *form, NMainFrameWidget *mainWidget) {
	mainWidget_ = mainWidget;
	staffList_ = stafflist;
	staffCount_ = stafflist->count();
	exportDialog_ = form;
	fileName = filen;
	this->doExport();
}

#define SEARCH_STAFF_NR(number, staff_address)  found = false; \
						number = 0; \
						while (!found && number < staffs_to_export) { \
							if (!(found = listOfStaffsToExport_[number] == staff_address)) number++; \
						} \
						if (!found) { \
							NResource::abort("SEARCH_STAFF_NR: internal error"); \
						}


void NMusiXTeX::doExport() {
	int i, j, k,maxlyrics;
	NExportError *exerr;
	NExportError *errptr;
	NPositStr *posit;
	QList<NPositStr> plist;
	char *notesString;
	bool withLyrics;
	char buffer[128+1];
	ostrstream os(buffer, 100);
	NStaff *staff_elem;
	NVoice *voice_elem;
	QList<QString> lyrNames;
	QString *lyrName;
	int myTime_ = 0;
	int num_positions, maxwidth, min_time = (1 << 30);
	int minlength;
	bool only_playables, staff_placed_elements;
	bool found;
	NKeySig *ksig;
	NClef *clef;
	NTimeSig *timesig;
	QString lyricslist[NUM_LYRICS];
	int count_of_lyrics;
	int kind, z;
	bool lyrcsLineCounts[NUM_LYRICS];
	bool somethingProduced;
	bool tex_special = false;
	int voice_count;
	int staffs_to_export;
	int real_staff_nr, staff_nr;

	slurTiePool_ = 0x0;
	trillPool_ = 0x0;
	beamPool_ = 0x0;
	barNr_ = 2;
	maxTies_ = 6;
	maxSlurs_ = 6;
	spare_ = false;
	maxFlags_ = 4;
	newTempo_ = -1;
	maxStaffsAndBeams_ = exportDialog_->texMusixadd->isChecked() ? 9 : 6;
	limitMeasures_ = exportDialog_->texMeasures_->isChecked();
	if (limitMeasures_) {
		maxMeasuresPerLine_ = exportDialog_->measureVal->value();
	}
	if (NResource::staffSelExport_ == 0) {
		NResource::staffSelExport_ = new bool[staffCount_];
		for (i = 0; i < (int) staffCount_; NResource::staffSelExport_[i++] = true);
		staffs_to_export = staffCount_;
	}
	else {
		staffs_to_export = 0;
		for (i = 0; i < (int) staffCount_; i++) {
			if (NResource::staffSelExport_[i]) {
				staffs_to_export++;
			}
		}
	}
	listOfStaffsToExport_ = new NStaff*[staffs_to_export];
	k = 0;
	if (NResource::staffSelExport_ == 0) {
		for (i = 0; i < (int) staffCount_; i++) {
			if (k >= staffs_to_export) {
				NResource::abort("NMusiXTeX::doExport :internal error", 1);
			}
			if ((listOfStaffsToExport_[k++] = staffList_->at(i)) == 0) {
				NResource::abort("NMusiXTeX::doExport :internal error", 2);
			}
		}
	}
	else {
		for (i = 0; i < (int) staffCount_; i++) {
			if (NResource::staffSelExport_[i]) {
				if (k >= staffs_to_export) {
					NResource::abort("NMusiXTeX::doExport :internal error", 3);
				}
				if ((listOfStaffsToExport_[k++] = staffList_->at(i)) == 0) {
					NResource::abort("NMusiXTeX::doExport :internal error", 4);
				}
			}
		}
	}
	trill_descr_ = new trill_descr_str[staffs_to_export];
	if (staffs_to_export > (int) maxStaffsAndBeams_) {
		os << "There are more than " << maxStaffsAndBeams_ << "Staffs (maximum = " << maxStaffsAndBeams_ << "). Please select some !" << '\0';
		KMessageBox::error(0, QString(os.str()), kapp->makeStdCaption(i18n("???")));
		return;
	}
	dysymDescr_ = new dynamics_descr_str[staffs_to_export];
	for (i = 0; i < staffs_to_export; i++) {
		trill_descr_[i].trill_nr = -1;
		dysymDescr_[i].lastDynSym = 0;
		dysymDescr_[i].dynEndPos = 0;
	}
	madeSeek_ = false;
	if (fileName.isNull())
		return;
	out_.open(fileName);
	if (!out_) {
		os << "error opening file " << fileName << '\0';
		KMessageBox::sorry(0, QString(os.str()), kapp->makeStdCaption(i18n("???")));
		return;
	}
	if (exportDialog_->texMusixbm->isChecked()) maxFlags_ = 5;
	lastBarNr_ = 0;
	for (i = 0; i < NUM_LYRICS; i++) {
		lyrcsLineCounts[i] = false;
	}
	for (i = 0, staff_elem = staffList_->first(); staff_elem; staff_elem = staffList_->next(), i++) {
		voice_elem = staff_elem->getVoiceNr(0);
		if (voice_elem->getLastBarNr() > lastBarNr_) lastBarNr_ = voice_elem->getLastBarNr();
		maxlyrics = voice_elem->countOfLyricsLines();
		if (maxlyrics < 0 || maxlyrics > NUM_LYRICS) {
			os << "internal error: maxlyrics = " << maxlyrics << '\0';
			KMessageBox::error(0, QString(os.str()), kapp->makeStdCaption(i18n("???")));
			return;
		}
		if (maxlyrics) lyrcsLineCounts[maxlyrics-1] = true;
	}
	badMeasures_.setAutoDelete(true);
	badMeasures_.clear();
	out_ << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
	out_ << "%                                  %" << endl;
	out_ << "% MusiXTeX output generated by     %" << endl;
	out_ << "%       \"noteedit\"                 %" << endl;
	out_ << "%                                  %" << endl;
	out_ << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl << endl;
	out_ << "\\documentclass{article}"  << endl;
	out_ << "\\usepackage{musixtex}" << endl << endl;
	out_ << "\\input musixper.tex" << endl << endl;
	out_ << "\\input musixlit.tex" << endl << endl;
	if (exportDialog_->texMusixadd->isChecked()) {
		out_ << "\\input musixadd.tex" << endl << endl;
	}
	if (exportDialog_->texMusixbm->isChecked()) {
		out_ << "\\input musixbm.tex" << endl << endl;
	}
	if (exportDialog_->texMLyr->isChecked()) {
		out_ << "\\input musixlyr.tex" << endl << endl;
	}
	out_ << "\\setlength{\\parindent}{20mm}" << endl;
	out_ << "\\setlength{\\textwidth}{" << exportDialog_->texWidth->text() << "mm}" << endl;
	out_ << "\\setlength{\\textheight}{" << exportDialog_->texHeight->text() << "mm}" << endl;
	out_ << "\\setlength{\\topmargin}{" << exportDialog_->texTop->text() << "mm}" << endl;
	out_ << "\\setlength{\\oddsidemargin}{" << exportDialog_->texLeft->text() << "mm}" << endl;
	if (exportDialog_->texSize->isChecked()) {
		out_ << "\\smallmusicsize" << endl;
	}
	if (!exportDialog_->texBar->isChecked()) {
		out_ << "\\nobarnumbers" << endl;
	}

	spare_ = exportDialog_->texTies->isChecked();
	out_ << "\\begin{document}" << endl << endl;

	lyrNames.setAutoDelete(true);
	for (i = 0, staff_elem = staffList_->first(); exportDialog_->texMLyr->isChecked() && staff_elem; staff_elem = staffList_->next(), i++) {
		if (!NResource::staffSelExport_[i]) continue;
		voice_elem = staff_elem->getVoiceNr(0);
		if (!(count_of_lyrics = voice_elem->countOfLyricsLines())) continue;
		voice_elem->collectLyrics(lyricslist);
		lyrNames.clear();
		for (j = 0; j < NUM_LYRICS; j++) {
			if (!lyricslist[j].isEmpty()) {
				lyrName = new QString();
				if (count_of_lyrics < 2) {
					lyrName->sprintf("lyrstaff%d", staffs_to_export - i);
				}
				else {
					lyrName->sprintf("lyrstaff%dverse%d", staffs_to_export - i, j+1);
				}
				lyrNames.append(lyrName);
				out_ << "\\setlyrics{" << (*lyrName) << "}{" << lyrics2TeX(&(lyricslist[j])) << "}" << endl;
			}
		}

		out_ << "\\assignlyrics{" << (staffs_to_export - i) << "}{";
		while(!lyrNames.isEmpty()) {
			out_ << *(lyrNames.first());
			lyrNames.remove();
			if (!lyrNames.isEmpty()) out_ << ',';
		}
		out_ << '}' << endl << endl;
	}


	if (!mainWidget_->scTitle_.isEmpty()) {
		if (!tex_special) {
			tex_special = true;
			out_ << "\\begin{center}" << endl;
		}
		out_ << "{\\Large\\bf " << lyrics2TeX(&(mainWidget_->scTitle_)) << "}\\\\" << endl;
	}
	if (!mainWidget_->scSubtitle_.isEmpty()) {
		if (!tex_special) {
			tex_special = true;
			out_ << "\\begin{center}" << endl;
		}
		out_ << "{\\Large " << lyrics2TeX(&(mainWidget_->scSubtitle_)) << "}\\\\" << endl;
	}
	if (tex_special) {
		out_ << "\\end{center}" << endl << endl;
		tex_special = false;
	}
	if (!mainWidget_->scAuthor_.isEmpty()) {
		if (!tex_special) {
			tex_special = true;
			out_ << "\\begin{flushright}" << endl;
		}
		out_ << "{\\Large\\sf " << lyrics2TeX(&(mainWidget_->scAuthor_)) << "}\\\\" << endl;
	}
	if (!mainWidget_->scLastAuthor_.isEmpty()) {
		if (!tex_special) {
			tex_special = true;
			out_ << "\\begin{flushright}" << "\\\\" << endl;
			out_ << "{\\large\\sf " <<  lyrics2TeX(&(mainWidget_->scLastAuthor_)) << "}\\\\" << endl;
		}
	}
	if (tex_special) {
		out_ << "\\end{flushright}" << endl << endl;
		tex_special = false;
	}
	out_ << "\\instrumentnumber{" << staffs_to_export << "}" << endl;
	k = 0;
	for (i = 0, staff_elem = staffList_->last(); staff_elem; staff_elem = staffList_->prev(), i++) {
		voice_elem = staff_elem->getVoiceNr(0);
		timesig = voice_elem->getFirstTimeSig();
		clef = voice_elem->getFirstClef();
		ksig = voice_elem->getFirstKeysig();
		if (NResource::staffSelExport_[staffCount_ - i - 1]) {
			k++;
			if (clef) {
				switch (clef->getSubType()) {
					case BASS_CLEF: out_ << "\\setclef{" << k << "}" << "{6}" << endl; break;
					case ALTO_CLEF: out_ << "\\setclef{" << k << "}" << "{3}" << endl; break;
					case TENOR_CLEF: out_ << "\\setclef{" << k << "}" << "{4}" << endl; break;
					case DRUM_BASS_CLEF:
					case DRUM_CLEF: out_ << "\\setclefsymbol{" << k << "}" << "\\drumclef" << endl; break;
					default: out_ << "\\setclef{" << k << "}" << "{0}" << endl; break;
				}
			}
			if (k > 1) {
				out_ << "\\setinterinstrument{" << k - 1 << "}{" << 
					TEX_FAC * voice_elem->getStaff()->underlength_ << "mm}" << endl;
			}
			else {
				out_ << "\\staffbotmarg=" << TEX_FAC * voice_elem->getStaff()->underlength_ << "mm%" << endl;
			}
			if (ksig) {
				if (ksig->isRegular(&kind, &z)) {
					if (z != 0) {
						out_ << "\\setsign{" << k << "}{" << ((kind == STAT_CROSS) ? z : -z) << "}" << endl;
					}
				}
				else {
					exerr = new NExportError(ERR_IRR_KEY, staffCount_ - i);
					badMeasures_.append(exerr);
				}
			}
			if (timesig) {
				out_ << "\\setmeter{" << k << "}{{\\meterfrac{" << timesig->getNumerator() <<
					"}{" << timesig->getDenominator() << "}}}" << endl;
			}
			if (!voice_elem->getStaff()->staffName_.isEmpty()) {
				out_ << "\\setname{" << k << "}{" << voice_elem->getStaff()->staffName_ << "}" << endl;
			}
			maxlyrics = voice_elem->countOfLyricsLines();
			voice_count = staff_elem->voiceCount();
			for (j = 0; j < voice_count; j++) {
				voice_elem = staff_elem->getVoiceNr(j);
				voice_elem->startRepositioning();
			}
		}
	}
	out_ << "\\startpiece" << endl;

	while(1) {
		plist.clear();
		withLyrics = false;
		num_positions = 0;
		min_time = (1 << 30);
		for (i = 0, staff_elem = staffList_->first(); staff_elem; staff_elem = staffList_->next(), i++) {
			if (!NResource::staffSelExport_[i]) continue;
			voice_count = staff_elem->voiceCount();
			for (j = voice_count - 1; j >= 0; j--) {
				voice_elem = staff_elem->getVoiceNr(j);
				posit = voice_elem->getElementAfter(myTime_);
				if (posit) {
					plist.append(posit); 
					num_positions++;
				}
				if (posit) {
					if (posit->ev_time < min_time) min_time = posit->ev_time;
					if (posit->elem->getType() == T_CHORD) {
						if (((NChord *) posit->elem)->getLyrics(0)) withLyrics = true;
					}
			
				}
			}
		}
		if (!num_positions) {
			break;
		}
		only_playables = true;
		staff_placed_elements = false;
		maxwidth = 0;
		minlength = 3*128;
		for (posit = plist.first(); posit; posit = plist.next()) {
			if (posit->ev_time != min_time) continue;
			if (!(posit->ev_type & PLAYABLE)) only_playables = false;
			if (posit->ev_width > maxwidth) maxwidth = posit->ev_width;
			switch(posit->elem->getType()) {
			case T_CHORD:
				if (((NChord *) posit->elem)->getLyrics(0)) withLyrics = true;
				break;
			case T_SIGN:
				if (posit->elem->getSubType() == VOLUME_SIG) staff_placed_elements = true;
				break;
			}
			if (posit->ev_type & PLAYABLE) {
				if (posit->elem->getMidiLength() < minlength) minlength = posit->elem->getMidiLength();
			}
		}
		if (withLyrics) {
			if (maxwidth < 16) notesString = "\\notes";
			else if (maxwidth < 18) notesString = "\\notesp";
			else if (maxwidth < 20) notesString = "\\Notes";
			else if (maxwidth < 22) notesString = "\\Notesp";
			else if (maxwidth < 24) notesString = "\\NOtes";
			else if (maxwidth < 26) notesString = "\\NOtesp";
			else if (maxwidth < 28) notesString = "\\NOTes";
			else if (maxwidth < 30) notesString = "\\NOTesp";
			else notesString = "\\NOTEs";
		}
		else {
			if (minlength >= 64 *3*3/2) notesString = "\\NOTEs";
			else if (minlength >= 64 *3) notesString = "\\NOTes";
			else if (minlength >= 32*3*3/2) notesString = "\\NOtesp";
			else if (minlength >= 32*3) notesString = "\\NOtes";
			else if (minlength >= 16*3*3/2) notesString = "\\Notesp";
			else if (minlength >= 16*3) notesString = "\\Notes";
			else if (minlength >= 8*3*3/2) notesString = "\\notesp";
			else notesString = "\\notes";
		}

		somethingProduced = false;
		staff_nr = staffs_to_export - 1;
		if (madeSeek_) {
			madeSeek_ = false;
			out_ << "\\changecontext %" << barNr_++ << endl;
		}
		if (only_playables || staff_placed_elements) {
			out_ << notesString;
		}
		for (posit = plist.last(); posit; posit = plist.prev()) {
			staff_elem = posit->from->getStaff();
			SEARCH_STAFF_NR(k, staff_elem);
			if (k != staff_nr) {
				somethingProduced = false;
				if (staff_nr < k) {
					NResource::abort("NMusiXTeX::doExport :internal error", 5);
				}
				while (staff_nr > k) {
					if (only_playables || staff_placed_elements) {
						out_ << '&';
					}
					staff_nr--;
				}
			}
			if ((real_staff_nr = staffList_->find(staff_elem)) == -1) {
					NResource::abort("NMusiXTeX::doExport :internal error", 6);
			}
			if (only_playables) {
				if (posit->ev_time == min_time)  {
					posit->from->skipTeXChord();
					generate(staff_nr, real_staff_nr, staffs_to_export, somethingProduced ? "\\bsk" : 0, posit->elem, staff_elem, posit->from);
					somethingProduced = true;
				}
			}
			else {
				if (posit->ev_time == min_time && !(posit->ev_type & PLAYABLE))   {
					posit->from->skipTeXChord();
					generate(staff_nr, real_staff_nr, staffs_to_export, somethingProduced ? "\\bsk" : 0, posit->elem, staff_elem, posit->from);
					somethingProduced = true;
				}
			}
		}
		if (staff_nr < 0) {
			NResource::abort("NMusiXTeX::doExport :internal error", 7);
		}
		if (only_playables || staff_placed_elements) {
			while (staff_nr > 0) {
				out_ << "&";
				staff_nr--;
			}
			out_ << "\\en" << endl;
		}
		if (only_playables) {
			myTime_ = min_time+1;
		}
	}

	if (lastSignWasBar_) {
		out_.seekp(lastBarPos_);
	}

	out_ << "\\Endpiece" << endl << endl;
	out_ << "\\end{document}" << endl;
	out_.close();
	delete listOfStaffsToExport_;
	delete trill_descr_;
	delete dysymDescr_;
	if (NResource::staffSelExport_) {
		delete [] NResource::staffSelExport_;
		NResource::staffSelExport_ = 0;
	}
	if (!badMeasures_.isEmpty()) {
		QString output;
		output = i18n
			("Noteedit has exported the score to MusiXTeX but there are some\n"
			 "constructs which cannot be expressed in MusiXTeX. The score will\n"
			 "work with MusiXTeX but Noteedit omitted these constructs.\n");
		output += i18n("-----------------------------------------------------\n");
		for (errptr = badMeasures_.first(); errptr; errptr = badMeasures_.next()) {
			switch (errptr->reason) {
				case ERR_128:
				output += i18n
					("128th in measure: %2; maximum = %3 flags\n").
					arg(errptr->measure).arg(maxFlags_);
					break;
				case ERR_IRR_KEY:
				output += i18n
					("Staff %1 has irregular keysig. This cannot be expressed in MusiXTeX\n").arg(errptr->measure);
					break;
				case ERR_TOO_MANY_TIES:
				output += i18n
					("Too many ties in measure: %2; maximum = %3\n").arg(errptr->measure).arg(maxTies_);
					break;
				case ERR_TOO_MANY_SLURS:
				output += i18n
					("Too many slurs in measure: %1, ; maximum = %2\n").arg(errptr->measure).arg(maxSlurs_);
					break;
				case ERR_TOO_MANY_TRILLS:
				output += i18n
					("Too many trills in measure: %1: maximum = 6\n").arg(errptr->measure);
					break;
				case ERR_NESTED_TILLS:
				output += i18n
					("Nested trills in measure: %2 ;\n").arg(errptr->measure);
					break;
				case ERR_BEAM_COUNT:
				output += i18n
					("too many beams in measure: %1; maximum: %2\n").arg(errptr->measure).arg(maxStaffsAndBeams_);
					break;
			}
		}
		if (NResource::commandLine_) {
			cerr << "MusiXTeX produced. But there are some problems" << endl;
			cerr << output << endl;
		}
		else {
			NResource::exportWarning_->setOutput(i18n ("MusiXTeX produced. But there are some problems."), &output);
			NResource::exportWarning_->show();
		}
	}
}

void NMusiXTeX::generate(int staff_nr, int real_staff_nr, int staffs_to_export, const char *extraDelimiter, NMusElement *elem, NStaff *staff_elem, NVoice *ac_voice) {
	QList<NNote> *notelist;
	NExportError *exerr;
	QString *s;
	char *bodyString;
	int lastBodyState;
	bool first, body_changed;
	QString stemdir;
	QString *lyrics;
	NNote *note, *note2;
	NTimeSig *timesig;
	NKeySig *ksig;
	QString musi_length;
	NNote *base_note;
	NChord *chord;
	QList<NNote> *notes_to_be_shifted, *notes_not_to_be_shifted;
	int countLyricsLines;
	bool found = false;
	bool shift_needed;
	bool toomany, nested, problem128;
	int num;
	int kind, z, i;
	int hline;
	QList<NNote> *acc_list;

	lastSignWasBar_ = false;
	switch(elem->getType()) {
		case T_CHORD:
/*
			     if (madeSeek_) {
				madeSeek_ = false;
				out_ << "\\changecontext %" << barNr_++ << endl;
			     }
*/
			     if (extraDelimiter) out_ << extraDelimiter;
			     chord = (NChord *) elem;
			     chord->initialize_acc_pos_computation();
			     if (staff_nr == 0 && newTempo_ >= 0) {
					out_ << "\\Uptext{\\metron{\\nq}{" << newTempo_ << "}}";
					newTempo_ = -1;
			     }
			     for (z = 0, i = chord->getNumOfTexAccRows() - 1; i >= 0; i--, z++) {
				acc_list = chord->getAccTexRow(i);
				out_ << "\\roff{";
			     	for (note = acc_list->first(); note; note = acc_list->next()) {
				   switch (note->offs) {
					case  1: out_ << "\\sh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case  0: out_ << "\\na " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case -1:  out_ << "\\fl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case  2:out_ << "\\dsh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case -2: out_ << "\\dfl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
				   }
				}
			     }
			     stemdir = ((elem->status_ & STAT_STEM_UP) ? QString("u") : QString("l"));
			     countLyricsLines = ((NChord *) elem)->countOfLyricsLines();
			     if (!exportDialog_->texMLyr->isChecked()) {
			     	for (i = countLyricsLines-1; i >= 0; i--) {
						lyrics = chord->getLyrics(i);
						if (lyrics) {
							out_ << "\\zcharnote{" << -8 - (int) ((staff_elem->staff_props_.lyricsdist - DEFAULT_LYRICSDIST ) * TEX_FAC) -i*4 << "}{"<< lyrics2TeX(lyrics) << "}";
						}
			     	}
			     }
			     s = elem->computeTeXTriplet(&(staff_elem->actualClef_));
			     if (s) {
				out_ << *s;
				delete s;
			     }
			     s = chord->computeTeXTie(&slurTiePool_, &(staff_elem->actualClef_), maxTies_, &toomany, spare_);
			     if (toomany) {
					exerr = new NExportError(ERR_TOO_MANY_TIES, barNr_-1);
					badMeasures_.append(exerr);
			     }
			     if (s) {
				out_ << *s;
				delete s;
			     }
			     s = chord->computeTeXSlur(&slurTiePool_, &(staff_elem->actualClef_), maxSlurs_, &toomany);
			     if (toomany) {
					exerr = new NExportError(ERR_TOO_MANY_SLURS, barNr_-1);
					badMeasures_.append(exerr);
			     }
			     if (s) {
				out_ << *s;
				delete s;
			     }
			     if (elem->status_ & STAT_BEAMED) {
				 s = chord->computeTeXBeam(maxStaffsAndBeams_, &beamPool_, &(ac_voice->beamNr_), &(ac_voice->beamCount_),
						 &(staff_elem->actualClef_), maxFlags_, &problem128, &toomany);
				 if (problem128) {
					exerr = new NExportError(ERR_128, barNr_-1);
					badMeasures_.append(exerr);
				 }
				 if (toomany) {
					exerr = new NExportError(ERR_BEAM_COUNT, barNr_-1);
					badMeasures_.append(exerr);
				 }
				 if (s) {
					out_ << *s;
					delete s;
				 }
			     }
			     notelist = chord->getNoteList();
			     if (trill_descr_[staff_nr].trill_nr >= 0 && 	
					chord->getBbox()->right() > trill_descr_[staff_nr].endpos) {
					out_ << "\\Ttrille" << trill_descr_[staff_nr].trill_nr;
					trillPool_ &= (~(1 << trill_descr_[staff_nr].trill_nr));
					trill_descr_[staff_nr].trill_nr = -1;
			     }
			     if (chord->trill_) {
				hline = ac_voice->findHighestLineInTrill(chord);
				s = chord->computeTeXTrill(hline, &trillPool_, &(staff_elem->actualClef_),
					&(trill_descr_[staff_nr]), &nested, &toomany);
				if (nested) {
					exerr = new NExportError(ERR_NESTED_TILLS, real_staff_nr + 1);
					badMeasures_.append(exerr);
				}
				if (toomany) {
					exerr = new NExportError(ERR_TOO_MANY_TRILLS, real_staff_nr + 1);
					badMeasures_.append(exerr);
				}
				if (s) {
					out_ << *s;
					delete s;
				}
			     }
			     if (chord->dynamic_) {
				dysymDescr_[staff_nr].lastDynSym = (char *) (chord->dynamicAlign_ ? "\\tcresc" : "\\tdecresc ");
				dysymDescr_[staff_nr].dynEndPos = chord->getDynamicEnd();
				out_ << "\\icresc ";
			     }
			     if (dysymDescr_[staff_nr].lastDynSym && 
				chord->getBbox()->right() > dysymDescr_[staff_nr].dynEndPos) {
				out_ << "\\zcharnote " << staff_elem->actualClef_.line2TexTab_[-5+LINE_OVERFLOW] << '{';
				out_ << dysymDescr_[staff_nr].lastDynSym << '}';
				dysymDescr_[staff_nr].lastDynSym = 0;
				dysymDescr_[staff_nr].dynEndPos = 0;
			     } 
#ifdef AAA
			     for (note = notelist->first(); note; note = notelist->next()) {
				if (note->status & STAT_FORCE) continue;
				switch (note->needed_acc) {
					case STAT_NO_ACC:break;
					case STAT_CROSS: out_ << "\\sh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case STAT_NATUR: out_ << "\\na " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case STAT_FLAT:  out_ << "\\fl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case STAT_DCROSS:out_ << "\\dsh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case STAT_DFLAT: out_ << "\\dfl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
				}
			     }
			     for (note = notelist->first(); note; note = notelist->next()) {
				if (!(note->status & STAT_FORCE)) continue;
				switch (note->offs) {
					case  1: out_ << "\\sh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case  0: out_ << "\\na " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case -1:  out_ << "\\fl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case  2:out_ << "\\dsh " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
					case -2: out_ << "\\dfl " << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
						      break;
				}
			     }
#endif
			     shiftes_notes_.clear(); non_shifted_notes_.clear(); shift_needed = false;
			     for (note = notelist->first(); note; note = notelist->next()) {
				if (note->status & STAT_SHIFTED) {
					shift_needed = true;
					shiftes_notes_.append(note);
				}
				else {
					non_shifted_notes_.append(note);
				}
			     }
			     if (shift_needed) {
			     	notes_to_be_shifted = (elem->status_ & STAT_STEM_UP) ? &shiftes_notes_ : &non_shifted_notes_;
			     	notes_not_to_be_shifted = (elem->status_ & STAT_STEM_UP) ? &non_shifted_notes_ : &shiftes_notes_;
			     }
			     else {
				notes_to_be_shifted = &shiftes_notes_;
			     	notes_not_to_be_shifted = &non_shifted_notes_;
			     }
			     base_note = notes_not_to_be_shifted->first();
			     if (shift_needed && !(elem->status_ & STAT_STEM_UP)) {
				out_ << "\\roff{";
			     }
			     if (notes_not_to_be_shifted->count() > 1) {
				lastBodyState = -1;
				for (first = true, note = notes_not_to_be_shifted->first(); note; note = notes_not_to_be_shifted->next()) {
					if (note == base_note) continue;
					body_changed = false;
					if (lastBodyState != (note->status & BODY_MASK)) {
						body_changed = true;
						switch (lastBodyState = (note->status & BODY_MASK)) {
							case STAT_BODY_CROSS: bodyString = "x"; break;
							case STAT_BODY_CROSS2: bodyString = "k"; break;
							case STAT_BODY_CIRCLE_CROSS: bodyString = "ox"; break;
							case STAT_BODY_RECT: bodyString = "ro"; break;
							case STAT_BODY_TRIA: bodyString = "tg"; break;
							default: bodyString = ""; break;
						}
					}
					if (body_changed || first) {
						if (!first) out_ << '}';
						switch (elem->getSubType()) {
							case 3*256: out_ << '\\' << bodyString << "zwq{"; break;
							case 3*128: out_ << '\\' << bodyString << "zw{"; break;
							case 3*64 : out_ << '\\' << bodyString << "zh{"; break;
							default   : out_ << '\\' << bodyString << "zq{"; break;
						}
					}
					switch(elem->status_ & DOT_MASK) {
						case 1: out_ << '.'; break;
						/*case 2:  don't know what to do ? ; break;*/
					}
					out_ << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
					first = false;
				}
				out_ << "}";
			     }
			     if (shift_needed) {
				out_ << ((elem->status_ & STAT_STEM_UP) ? "\\roff{" : "\\loff{"); 
				lastBodyState = -1;
				for (first = true, note = notes_to_be_shifted->first() ;note ; note = notes_to_be_shifted->next(), first = false) {
					body_changed = false;
					if (lastBodyState != (note->status & BODY_MASK)) {
						body_changed = true;
						switch (lastBodyState = (note->status & BODY_MASK)) {
							case STAT_BODY_CROSS: bodyString = "x"; break;
							case STAT_BODY_CROSS2: bodyString = "k"; break;
							case STAT_BODY_CIRCLE_CROSS: bodyString = "ox"; break;
							case STAT_BODY_RECT: bodyString = "ro"; break;
							case STAT_BODY_TRIA: bodyString = "tg"; break;
							default: bodyString = ""; break;
						}
					}
					if (body_changed || first) {
						if (!first) out_ << '}';
						switch (elem->getSubType()) {
							case 3*256: out_ << '\\' << bodyString << "zwq{"; break;
							case 3*128: out_ << '\\' << bodyString << "zw{"; break;
							case 3*64 : out_ << '\\' << bodyString << "zh{"; break;
							default   : out_ << '\\' << bodyString << "zq{"; break;
						}
					}
					switch(elem->status_ & DOT_MASK) {
						case 1: out_ << '.'; break;
						/*case 2:  don't know what to do ? ; break;*/
					}
					out_ << staff_elem->actualClef_.line2TexTab_[note->line+LINE_OVERFLOW];
				}
				out_ << "}}";
			     }
			     switch (base_note->status & BODY_MASK) {
				case STAT_BODY_CROSS: bodyString = "x"; break;
				case STAT_BODY_CROSS2: bodyString = "k"; break;
				case STAT_BODY_CIRCLE_CROSS: bodyString = "ox"; break;
				case STAT_BODY_RECT: bodyString = "ro"; break;
				case STAT_BODY_TRIA: bodyString = "tg"; break;
				default: bodyString = ""; break;
			     }
			     if (elem->status_ & STAT_BEAMED && ac_voice->beamNr_ >= 0) {
				musi_length.sprintf("%sqb%d", bodyString, ac_voice->beamNr_);
			     }
			     else switch (elem->getSubType()) {
				case 3*256: musi_length = QString("wq"); break;
				case 3*128: musi_length = QString(bodyString) + QString("wh"); break;
				case 3*64 : musi_length = QString(bodyString) + QString("h")+stemdir; break;
				case 3*32 : musi_length = QString(bodyString) + QString("q")+stemdir; break;
				case 3*16 : musi_length = QString(bodyString) + QString("c")+stemdir; break;
				case 3*8  : musi_length = QString(bodyString) + QString("cc")+stemdir; break;
				case 3*4  : musi_length = QString(bodyString) + QString("ccc")+stemdir; break;
				case 3*2  : musi_length = QString(bodyString) + QString("cccc")+stemdir; break;
				case 3*1  : musi_length = QString(bodyString) + QString("cccc")+stemdir;
					    if (maxFlags_ < 5) {
							exerr = new NExportError(ERR_128, barNr_-1);
									badMeasures_.append(exerr);
					    }
					    else {
						musi_length = QString(bodyString) + QString("ccccc")+stemdir;
					    }
					    break;
			     }
			     switch(elem->status_ & DOT_MASK) {
				case 1: out_ << "\\pt " << staff_elem->actualClef_.line2TexTab_[base_note->line + LINE_OVERFLOW]; break;
				case 2: out_ << "\\ppt " <<  staff_elem->actualClef_.line2TexTab_[base_note->line + LINE_OVERFLOW]; break;
			     }
			     if (elem->status_ & STAT_STACC || elem->status_ & STAT_SFORZ ||
			         elem->status_ & STAT_PORTA || elem->status_ & STAT_STPIZ ||
				 elem->status_ & STAT_SFZND) {
				out_ << '\\';
				out_ << ((elem->status_ & STAT_STEM_UP) ? "l" : "u");
				if (elem->status_ & STAT_STACC) out_ << "pz";
				if (elem->status_ & STAT_SFORZ) out_ << "sfz";
				if (elem->status_ & STAT_PORTA) out_ << "st";
				if (elem->status_ & STAT_STPIZ) out_ << "ppz";
				if (elem->status_ & STAT_SFZND) out_ << "sf";
				out_ << " " << staff_elem->actualClef_.line2TexTab_[base_note->line+LINE_OVERFLOW];
			     }
			     
			     if(elem->status_ & STAT_FERMT) {
			     	    note2 = (elem->status_ & STAT_STEM_UP) ? notelist->first() :  notelist->last();
			    	    out_ << "\\fermata" << ((elem->status_ & STAT_STEM_UP) ? "down" : "up")
				         << " "
				         << staff_elem->actualClef_.line2TexTab_[note2->line+LINE_OVERFLOW];
				    }
			     
			     out_ << "\\" << musi_length << ' ';
			     out_ << staff_elem->actualClef_.line2TexTab_[base_note->line+LINE_OVERFLOW];
			     if (shift_needed && !(elem->status_ & STAT_STEM_UP)) {
				out_ << '}';
			     }
			     for (i = chord->getNumOfTexAccRows() - 1; i >= 0; i--) {
				out_ << '}';
			     }
			     break;
		case T_REST: if (elem->status_ & STAT_HIDDEN) break;
/*
			     if (madeSeek_) {
				madeSeek_ = false;
				out_ << "\\changecontext %" << barNr_++ << endl;
			     }
*/
			     if (extraDelimiter) out_ << extraDelimiter;
			     if (staff_nr == 0 && newTempo_ >= 0) {
					out_ << "\\Uptext{\\metron{\\nq}{" << newTempo_ << "}}";
					newTempo_ = -1;
			     }
			     s = elem->computeTeXTriplet(&(staff_elem->actualClef_));
			     if (s) {
				out_ << *s;
				delete s;
			     }
			     switch(elem->status_ & DOT_MASK) {
				case 1: out_ << "\\pt " <<  staff_elem->actualClef_.line2TexTab_[4 + LINE_OVERFLOW - 2*ac_voice->yRestOffs_]; break;
				case 2: out_ << "\\ppt " <<  staff_elem->actualClef_.line2TexTab_[4 + LINE_OVERFLOW - 2*ac_voice->yRestOffs_]; break;
			     }
			     if (ac_voice->yRestOffs_) {
			     	out_ << "\\raise" << -ac_voice->yRestOffs_ << "\\Interligne"; 
			     }
			     switch (elem->getSubType()) {
				case 3*128: musi_length = QString("pause"); break;
				case 3*64 : musi_length = QString("hpause"); break;
				case 3*32 : musi_length = QString("qp"); break;
				case 3*16 : musi_length = QString("ds"); break;
				case 3*8  : musi_length = QString("qs"); break;
				case 3*4  : musi_length = QString("hs"); break;
				case 3*2  : musi_length = QString("qqs"); break;
				case 3*1  : musi_length = QString("qqs");
					    if (maxFlags_ < 5) {
							exerr = new NExportError(ERR_128, barNr_-1);
									badMeasures_.append(exerr);
					    }
					    break;
			     }
			     if (elem->getSubType() == MULTIREST)
				out_ << "\\Hpause32\\uptext{\\sk\\meterfont{" << ((NRest *)elem)->getMultiRestLength() << "}}\\sk\\sk\\hsk";
			     else 
			        out_ << "\\" << musi_length;
			     break;
		case T_SIGN: /* Don't know ??? if (madeSeek_) {
				madeSeek_ = false;
				out_ << "\\changecontext %" << barNr_++ << endl;
			     }
			     */
			     switch (elem->getSubType()) {
				case SIMPLE_BAR: if (staff_nr == 0) {
							if (ac_voice->testSpecialEnding(&num)) {
								out_ << "\\setvolta" << num;
							}
							lastBarPos_ = out_.tellp();
							if (limitMeasures_ && ((barNr_ - 1) % maxMeasuresPerLine_) == 0 &&
								barNr_ != lastBarNr_) {
								 out_ << "\\alaligne %" << barNr_++ << endl;
							}
							else {
								out_ << "\\bar % "  << barNr_++ << endl;
							}
						 	lastSignWasBar_ = true;
						 }
						 break;
				case REPEAT_OPEN: if (staff_nr == 0) {
							if (ac_voice->testSpecialEnding(&num)) {
								out_ << "\\setvolta" << num;
							}
							out_ << "\\leftrepeat % " << barNr_++ << endl;
							lastBarPos_ = out_.tellp();
						   }
						break;
				case REPEAT_CLOSE: if (staff_nr == 0) {
							if (ac_voice->testSpecialEnding(&num)) {
								out_ << "\\setvolta" << num;
							}
							out_ << "\\rightrepeat % " << barNr_++ << endl;
							lastBarPos_ = out_.tellp();
						 	lastSignWasBar_ = true;
						   }
						break;
				case REPEAT_OPEN_CLOSE: if (staff_nr == 0) {
							if (ac_voice->testSpecialEnding(&num)) {
								out_ << "\\setvolta" << num;
							}
							out_ << "\\leftrightrepeat % " << barNr_++ << endl;
							lastBarPos_ = out_.tellp();
						   }
						break;
				case TEMPO_SIGNATURE: 
						   if (!exportDialog_->texTempo->isChecked()) break;
						   newTempo_ = ((NSign *) elem)->getTempo();
						break;
				case VOLUME_SIG: if (!exportDialog_->texVolume->isChecked()) break;
						out_ << "\\Uptext";
							switch(((NSign *) elem)->getVolType()) {
								case V_PPPIANO : out_ << "\\ppp"; break;
								case V_PPIANO  : out_ << "\\pp"; break;
								case V_PIANO   : out_ << "\\p"; break;
								case V_FORTE   : out_ << "\\f"; break;
								case V_FFORTE  : out_ << "\\ff"; break;
								case V_FFFORTE : out_ << "\\fff"; break;
								default        : out_ << "\\mf"; break;
							}
						break;
			     }
			     break;
		case T_TIMESIG:
			     if (barNr_ <= 2) break;
			     timesig = (NTimeSig *) elem;
			     if (!madeSeek_) {
				madeSeek_ = true;
				out_.seekp(lastBarPos_); 
				barNr_--;
			     }
			     out_ << "\\setmeter{" << (staff_nr + 1) << "}{{\\meterfrac{" << timesig->getNumerator() <<
					"}{" << timesig->getDenominator() << "}}}%" << endl;
			     break;
		case T_KEYSIG:
			     if (barNr_ <= 2) break;
			     ksig = (NKeySig *) elem;
			     if (ksig->isRegular(&kind, &z)) {
				if (z != 0 || barNr_ > 1) {
				        if (!madeSeek_) {
						madeSeek_ = true;
						out_.seekp(lastBarPos_); 
						barNr_--;
				        }
					out_ << "\\setsign{" <<  (staffs_to_export - staff_nr) << "}{" << ((kind == STAT_CROSS) ? z : -z) << "}%" << endl;
				}
			     }
			     break;
		case T_CLEF:
			     staff_elem->actualClef_.change((NClef *) elem);
			     switch(elem->getSubType()) {
				case BASS_CLEF:
			     		if (barNr_ <= 2) break;
			     		if (!madeSeek_) {
						madeSeek_ = true;
						out_.seekp(lastBarPos_); 
						barNr_--;
			     		}
			     		out_ << "\\setclef{" << (staffs_to_export - staff_nr) << "}" << "{6}%" << endl;
			     		break;
				case TREBLE_CLEF:
			     		if (barNr_ <= 2) break;
			     		if (!madeSeek_) {
						madeSeek_ = true;
						out_.seekp(lastBarPos_); 
						barNr_--;
			     		}
			     		out_ << "\\setclef{" << (staffs_to_export - staff_nr) << "}" << "{0}%" << endl;
			     		break;
				case ALTO_CLEF:
			     		if (barNr_ <= 2) break;
			     		if (!madeSeek_) {
						madeSeek_ = true;
						out_.seekp(lastBarPos_); 
						barNr_--;
			     		}
			     		out_ << "\\setclef{" << (staffs_to_export - staff_nr) << "}" << "{3}%" << endl;
			     		break;
				case TENOR_CLEF:
			     		if (barNr_ <= 2) break;
			     		if (!madeSeek_) {
						madeSeek_ = true;
						out_.seekp(lastBarPos_); 
						barNr_--;
			     		}
			     		out_ << "\\setclef{" << (staffs_to_export - staff_nr) << "}" << "{4}%" << endl;
			     		break;
			     }
			     break;
	}
}

QString NMusiXTeX::lyrics2TeX(QString *lyrics) {
	QString ret;
	QRegExp reg;

	ret = QString(*lyrics);
	reg = QRegExp("");
	ret.replace (reg, "\\\"a");
	reg = QRegExp("");
	ret.replace (reg, "\\\"o");
	reg = QRegExp("");
	ret.replace (reg, "\\\"u");
	reg = QRegExp("");
	ret.replace (reg, "\\\"A");
	reg = QRegExp("");
	ret.replace (reg, "\\\"O");
	reg = QRegExp("");
	ret.replace (reg, "\\\"U");
	reg = QRegExp("");
	ret.replace (reg, "\\\"s");
	if (exportDialog_->texMLyr->isChecked()) {
		reg = QRegExp("<");
		ret.replace (reg, "{");
		reg = QRegExp(">");
		ret.replace (reg, "}");
	}
	else {
		reg = QRegExp("[<>]");
		ret.replace (reg, "");
	}
	return ret;
}
