/*
	TuxPuck - aiplayers.cpp
	Copyright (C) 2001 Jacob Kroon

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	Jacob Kroon - d00jkr@efd.lth.se
*/

/* includes */
#include <stdlib.h>
#include "video.h"
#include "aiplayers.h"

/* externals */
extern unsigned char tux_png[];
extern unsigned char tux_eyes_spr[];
extern unsigned char tux_boos_ogg[];
extern unsigned char tux_apps_ogg[];
extern unsigned char cartman_png[];
extern unsigned char cartman_eyes_spr[];
extern unsigned char cartman_mouth_spr[];
extern unsigned char cartman_kickass_ogg[];
extern unsigned char cartman_soab_ogg[];
extern unsigned char homer_png[];
extern unsigned char homer_eyes_spr[];
extern unsigned char homer_mouth_spr[];
extern unsigned char homer_doh_ogg[];
extern unsigned char homer_woohoo_ogg[];

/* TuxPlayer implemention */
TuxPlayer::TuxPlayer(Pad * pad, Puck * puck):AIPlayer(pad, puck, "Tux")
{
  _sdlTux = Video::instance()->createSurface(tux_png, NULL);
  _sprEyes = new Sprite(tux_eyes_spr, NULL);
  _sprEyes->setPosition(292, 47);
  _sndApps = Audio::instance()->createSound(tux_apps_ogg, NULL);
  _sndBoos = Audio::instance()->createSound(tux_boos_ogg, NULL);
  if (_sndApps)
    _sndApps->single = 0;
  if (_sndBoos)
    _sndBoos->single = 0;
  _speed = 3.5;
  _hitPower = 3.5;
}

TuxPlayer::~TuxPlayer()
{
  SDL_FreeSurface(_sdlTux);
  delete(_sprEyes);
  Audio::instance()->freeSound(_sndApps);
  Audio::instance()->freeSound(_sndBoos);
}

void TuxPlayer::setState(Uint8 state)
{
  _state = state;
  switch (_state)
    {
    case PLAYER_STATE_WIN_POINT:
      Audio::instance()->playSound(_sndApps);
      _sprEyes->setAnimation(1);
      break;
    case PLAYER_STATE_LOOSE_POINT:
      Audio::instance()->playSound(_sndBoos);
      _sprEyes->setAnimation(1);
      break;
    default:
      break;
    }
}

void TuxPlayer::blit(void)
{
  static SDL_Rect rect =
    { 320 - _sdlTux->w / 2, (Sint16) A2 - _sdlTux->h, 0, 0 };
  Video::instance()->blit(_sdlTux, &rect);
}

Uint8 TuxPlayer::reblit(Uint32 time)
{
  switch (_state)
    {
    case PLAYER_STATE_WIN_POINT:
    case PLAYER_STATE_LOOSE_POINT:
      _sprEyes->blit();
      if (_sprEyes->update(time) == 0)
	{
	  _sprEyes->erase();
	  _state = PLAYER_STATE_IDLE;
	}
      break;
    case PLAYER_STATE_WIN_GAME:
    case PLAYER_STATE_LOOSE_GAME:
      _state = PLAYER_STATE_IDLE;
      return 0;
    default:
      break;
    }
  return 1;
}

/* HomerPlayer implemention */
HomerPlayer::HomerPlayer(Pad * pad, Puck * puck):AIPlayer(pad, puck, "Homer")
{
  _sdlHomer = Video::instance()->createSurface(homer_png, NULL);
  _sndDoh = Audio::instance()->createSound(homer_doh_ogg, NULL);
  _sndWoohoo = Audio::instance()->createSound(homer_woohoo_ogg, NULL);
  _sprEyes = new Sprite(homer_eyes_spr, NULL);
  _sprMouth = new Sprite(homer_mouth_spr, NULL);
  _sprEyes->setPosition(303, 53);
  _sprMouth->setPosition(312, 91);
  _speed = 1.5;
  _hitPower = 1.5;
}

HomerPlayer::~HomerPlayer()
{
  SDL_FreeSurface(_sdlHomer);
  Audio::instance()->freeSound(_sndDoh);
  Audio::instance()->freeSound(_sndWoohoo);
  delete(_sprEyes);
  delete(_sprMouth);
}

void HomerPlayer::setState(Uint8 state)
{
  float odds;

  _state = state;
  odds = (float) rand() / RAND_MAX;
  switch (_state)
    {
    case PLAYER_STATE_LOOSE_POINT:
      _sprEyes->setAnimation(3);
      _sprMouth->setAnimation(1);
      if (odds > 0.60)
	{
	  Audio::instance()->playSound(_sndDoh);
	  _sprMouth->setAnimation(2);
	  _sprEyes->setAnimation(2);
	}
      break;
    case PLAYER_STATE_WIN_POINT:
      _sprEyes->setAnimation(1);
      _sprMouth->setAnimation(0);
      if (odds > 0.5)
	{
	  Audio::instance()->playSound(_sndWoohoo);
	  _sprMouth->setAnimation(3);
	}
      break;
    case PLAYER_STATE_LOOSE_GAME:
      _sprEyes->setAnimation(3);
      _sprMouth->setAnimation(1);
    default:
      break;
    }
}

void HomerPlayer::blit(void)
{
  static SDL_Rect rect =
    { 320 - _sdlHomer->w / 2, (Sint16) A2 - _sdlHomer->h, 0, 0 };
  Video::instance()->blit(_sdlHomer, &rect);
}

Uint8 HomerPlayer::reblit(Uint32 time)
{
  switch (_state)
    {
    case PLAYER_STATE_WIN_POINT:
      _sprEyes->blit();
      _sprMouth->blit();
      if (_sprMouth->update(time) == 0 || _sprEyes->update(time) == 0)
	{
	  _sprEyes->erase();
	  _sprMouth->erase();
	  _state = PLAYER_STATE_IDLE;
	}
      break;
    case PLAYER_STATE_LOOSE_POINT:
      _sprEyes->blit();
      _sprMouth->blit();
      if (_sprMouth->update(time) == 0 || _sprEyes->update(time) == 0)
	{
	  _sprEyes->erase();
	  _sprMouth->erase();
	  _state = PLAYER_STATE_IDLE;
	}
      break;
    case PLAYER_STATE_WIN_GAME:
      return 0;
    case PLAYER_STATE_LOOSE_GAME:
      _sprMouth->blit();
      _sprEyes->blit();
      if (_sprEyes->update(time) == 0 || _sprMouth->update(time) == 0)
	{
	  _sprMouth->erase();
	  _sprEyes->erase();
	  _state = PLAYER_STATE_IDLE;
	  return 0;
	}
      break;
    default:
      break;
    }
  return 1;
}

/* CartmanPlayer implemention */
CartmanPlayer::CartmanPlayer(Pad * pad, Puck * puck):AIPlayer(pad, puck,
	 "Cartman")
{
  _sdlCartman = Video::instance()->createSurface(cartman_png, NULL);
  _sndKickAss = Audio::instance()->createSound(cartman_kickass_ogg, NULL);
  _sndSoab = Audio::instance()->createSound(cartman_soab_ogg, NULL);
  _sprEyes = new Sprite(cartman_eyes_spr, NULL);
  _sprMouth = new Sprite(cartman_mouth_spr, NULL);
  _sprEyes->setPosition(290, 87);
  _sprMouth->setPosition(300, 128);
  _speed = 2.0;
  _hitPower = 2.0;
}

CartmanPlayer::~CartmanPlayer()
{
  SDL_FreeSurface(_sdlCartman);
  delete(_sprEyes);
  delete(_sprMouth);
  Audio::instance()->freeSound(_sndKickAss);
  Audio::instance()->freeSound(_sndSoab);
}

void CartmanPlayer::setState(Uint8 state)
{
  _state = state;
  float odds = 0;

  switch (_state)
    {
    case PLAYER_STATE_WIN_POINT:
      odds = (float) rand() / RAND_MAX;
      if (odds > 0.60)
	{
	  Audio::instance()->playSound(_sndKickAss);
	  _sprMouth->setAnimation(1);
	}
      break;
    case PLAYER_STATE_LOOSE_POINT:
      _sprEyes->setAnimation(1);
      break;
    case PLAYER_STATE_LOOSE_GAME:
      _sprEyes->setAnimation(2);
      _sprMouth->setAnimation(2);
      Audio::instance()->playSound(_sndSoab);
      break;
    default:
      break;
    }
}

void CartmanPlayer::blit(void)
{
  static SDL_Rect rect =
    { 320 - _sdlCartman->w / 2, (Sint16) A2 - _sdlCartman->h, 0, 0 };
  Video::instance()->blit(_sdlCartman, &rect);
}

Uint8 CartmanPlayer::reblit(Uint32 time)
{
  switch (_state)
    {
    case PLAYER_STATE_WIN_POINT:
      _sprMouth->blit();
      if (_sprMouth->update(time) == 0)
	{
	  _sprMouth->erase();
	  _state = PLAYER_STATE_IDLE;
	}
      break;
    case PLAYER_STATE_LOOSE_POINT:
      _sprEyes->blit();
      if (_sprEyes->update(time) == 0)
	{
	  _sprEyes->erase();
	  _state = PLAYER_STATE_IDLE;
	}
      break;
    case PLAYER_STATE_LOOSE_GAME:
      _sprEyes->blit();
      _sprMouth->blit();
      if (_sprMouth->update(time) == 0 || _sprEyes->update(time) == 0)
	{
	  _sprEyes->erase();
	  _sprMouth->erase();
	  _state = PLAYER_STATE_IDLE;
	  return 0;
	}
      break;
    case PLAYER_STATE_WIN_GAME:
      return 0;
    default:
      break;
    }
  return 1;
}
