/*
	TuxPuck - player.h
	Copyright (C) 2001 Jacob Kroon

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	Jacob Kroon - d00jkr@efd.lth.se
*/

#ifndef _PLAYER_H
#define _PLAYER_H

class Player;
class HumanPlayer;
class AIPlayer;

/* includes */
#ifndef _BOARDENTITY_H
#include "boardentity.h"
#endif

/* defines */
#define PLAYER_STATE_IDLE       1
#define PLAYER_STATE_AIM        2
#define PLAYER_STATE_HIT        3
#define PLAYER_STATE_BACKUP     4
#define PLAYER_STATE_WIN_POINT  5
#define PLAYER_STATE_LOOSE_POINT 6
#define PLAYER_STATE_WIN_GAME	7
#define PLAYER_STATE_LOOSE_GAME	8

/* class */
class Player
{
private:
  Uint8 _points;
  char *_name;
protected:
    Pad * _thePad;
  float _speed;
public:
    Player(Pad *, char *);
  void givePoint(void);
  Uint8 getPoints(void);
  char *getName(void);
    virtual ~ Player();
  virtual void update(Uint32) = 0;
};

class HumanPlayer:public Player
{
private:
public:
  HumanPlayer(Pad *, char *);
   ~HumanPlayer();
  void setSpeed(Uint8);
  void update(Uint32);
};

class AIPlayer:public Player
{
private:
  Puck * _thePuck;
protected:
  float _hitPower;
  Uint8 _state;
public:
    AIPlayer(Pad *, Puck *, char *);
   ~AIPlayer();
  void update(Uint32);
  virtual void setState(Uint8) = 0;
  virtual Uint8 reblit(Uint32) = 0;
  virtual void blit(void) = 0;
};

#endif /* _PLAYER_H */
