/*
	TuxPuck - video.cpp
	Copyright (C) 2001 Jacob Kroon

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	Jacob Kroon - d00jkr@efd.lth.se
*/

/* includes */
#include <stdlib.h>
#include <SDL_video.h>
#include <SDL_mouse.h>
#include <SDL_error.h>
#include "video.h"

/* defines */
#define MAX_RECTS	500

/* externals */
extern "C"
{
  void error(char *);
  SDL_Surface *loadPNG(Uint8 *, Uint32 *);
  SDL_Surface *zoomSurface(SDL_Surface *, float);
}

/* Video implemention */
Video::Video()
{
  if ((_screen = SDL_SetVideoMode(640, 480, 0, SDL_SWSURFACE | SDL_HWSURFACE)) == NULL)	/* Set video mode */
    error(SDL_GetError());
  SDL_WM_SetCaption("TuxPuck v" _VERSION, "TuxPuck v" _VERSION);     /* Set window captions */
  SDL_ShowCursor(0);
  _background = NULL;
  _noDirtyRects = 0;
  _dirtyRect = (SDL_Rect *) malloc(MAX_RECTS * sizeof(SDL_Rect));
}

Video::~Video()
{
  if (_background != NULL)
    SDL_FreeSurface(_background);
  free(_dirtyRect);
  _instance = NULL;
}

SDL_Surface *Video::createSurface(Uint8 * data, Uint32 * memcounter)
{
  return loadPNG(data, memcounter);
}

SDL_Surface *Video::scaleSurface(SDL_Surface * surface, float scale)
{
  return zoomSurface(surface, scale);
}

void Video::update(void)
{
  SDL_UpdateRects(_screen, _noDirtyRects, _dirtyRect);
  _noDirtyRects = 0;
}

void Video::erase(SDL_Rect * rect)
{
  SDL_BlitSurface(_background, rect, _screen, rect);
  _dirtyRect[_noDirtyRects++] = *rect;
}

void Video::fill(Uint32 color, SDL_Rect * rect)
{
  SDL_Rect rect2;

  SDL_FillRect(_screen, rect, color);
  if (!rect)
    {
      rect2.x = rect2.y = 0;
      rect2.w = 640;
      rect2.h = 480;
      rect = &rect2;
    }
  _dirtyRect[_noDirtyRects++] = *rect;
}

void Video::blend(Uint8 alpha, SDL_Rect * rect)
{
  SDL_Surface *blend = NULL;

  blend =
    SDL_CreateRGBSurface(SDL_SWSURFACE, rect->w, rect->h, 8, 0, 0, 0, 0);
  SDL_FillRect(blend, NULL, 0);
  SDL_SetAlpha(blend, SDL_SRCALPHA | SDL_RLEACCEL, alpha);
  this->blit(blend, rect);
  SDL_FreeSurface(blend);
}

void Video::erase(SDL_Surface * eraser, SDL_Rect * area)
{
  SDL_BlitSurface(eraser, area, _screen, area);
  _dirtyRect[_noDirtyRects++] = *area;
}

void Video::blit(SDL_Surface * surface, SDL_Rect * rect)
{
  SDL_Rect ownRect;

  if (!rect)
    {
      ownRect.x = ownRect.y = 0;
      rect = &ownRect;
    }
  SDL_BlitSurface(surface, NULL, _screen, rect);
  _dirtyRect[_noDirtyRects++] = *rect;
}

void Video::restore(void)
{
  SDL_BlitSurface(_background, NULL, _screen, NULL);
  SDL_UpdateRect(_screen, 0, 0, 0, 0);
  _noDirtyRects = 0;
}

void Video::save(void)
{
  if (_background != NULL)
    SDL_FreeSurface(_background);
  _background =
    SDL_ConvertSurface(_screen, _screen->format,
		       SDL_SWSURFACE | SDL_HWSURFACE);
}

SDL_Surface *Video::duplicate(void)
{
  SDL_Surface *screen = NULL;

  screen =
    SDL_ConvertSurface(_screen, _screen->format,
		       SDL_SWSURFACE | SDL_HWSURFACE);
  return screen;
}

void Video::toggleFullscreen(void)
{
  SDL_WM_ToggleFullScreen(_screen);
}

Video *Video::instance(void)
{
  if (_instance == NULL)
    _instance = new Video();
  return _instance;
}

Video *Video::_instance = NULL;
